using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Xml;

using Microsoft.BizTalk.ExplorerOM;
using Microsoft.BizTalk.RuleEngineExtensions;

using Microsoft.RuleEngine;

namespace BizTalkFactory.Management.Automation
{
    public class BtsApplication : BtsArtifact
    {
        private Application application_;

        private BtsApplication(BtsCatalog catalog, Application application)
            : base(catalog)
        {
            application_ = application;
        }

        public static BtsApplication FromItem(BtsCatalog catalog, object item)
        {
            return new BtsApplication(catalog, item as Application);
        }

        #region Attributes

        public BtsBackReferenceCollection BackReferences
        {
            get { return new BtsBackReferenceCollection(Catalog, application_); }
        }

        public string Description
        {
            get { return application_.Description; }
            set { application_.Description = value; }
        }

        public bool IsConfigured
        {
            get { return application_.IsConfigured; }
        }

        public bool IsDefaultApplication
        {
            get { return application_.IsDefaultApplication; }
        }

        public bool IsSystem
        {
            get { return application_.IsSystem; }
        }

        public override string Name
        {
            get { return application_.Name; }
        }

        public void SetName(string name)
        {
            application_.Name = name;
        }

        public BtsReferenceCollection References
        {
            get { return new BtsReferenceCollection(Catalog, application_); }
        }

        public BtsApplicationStatus Status
        {
            get { return ApplicationStatusConverter.FromNativeType(application_.Status); }
        }

        #endregion

        #region Bindings

        public void ExportBinding(string destination, bool includeGlobalPartyBinding)
        {
            Catalog.ExportBinding(this, destination, includeGlobalPartyBinding);
        }

        public void ImportBinding(string source)
        {
            Catalog.ImportBinding(this, source);
        }
        
        #endregion

        #region Operations

        public void AddReference(BtsApplication reference)
        {
            application_.AddReference(Catalog.BtsCatalogExplorer.Applications[reference.Name]);
        }

        public void RemoveReference(BtsApplication reference)
        {
            application_.RemoveReference(Catalog.BtsCatalogExplorer.Applications[reference.Name]);
        }

        public void Start()
        {
            Start(BtsApplicationStartOption.StartAll & ~BtsApplicationStartOption.StartReferencedApplications);
        }

        public void Start(BtsApplicationStartOption StartOption)
        {
            application_.Start(ApplicationStartOptionConverter.ToNativeType(StartOption));
        }

        public void Stop()
        {
            Stop(BtsApplicationStopOption.StopAll &~ BtsApplicationStopOption.StopReferencedApplications);
        }

        public void Stop(BtsApplicationStopOption StopOption)
        {
            application_.Stop(ApplicationStopOptionConverter.ToNativeType(StopOption));
        }

        #endregion

        #region Orchestrations

        public void RemoveOrchestration(string name)
        {
            RemoveOrchestration(Orchestrations[name]);
        }

        public void RemoveOrchestration(BtsOrchestration orchestration)
        {
            RemoveResource("System.BizTalk:BizTalkAssembly", orchestration.AssemblyQualifiedName);
        }

        public BtsOrchestrationCollection Orchestrations
        {
            get { return new BtsOrchestrationCollection(this); }
        }

        #endregion

        #region Pipelines

        public void RemovePipeline(string name)
        {
            RemovePipeline(Pipelines[name]);
        }

        public void RemovePipeline(BtsPipeline pipeline)
        {
            RemoveResource("System.BizTalk:BizTalkAssembly", pipeline.AssemblyQualifiedName);
        }

        public BtsPipelineCollection Pipelines
        {
            get { return new BtsPipelineCollection(this); }
        }

        #endregion

        #region Policies

        public void ExportPolicy(BtsPolicy policy, string destination)
        {
            ExportPolicy(policy, destination, true);
        }

        public void ExportPolicy(BtsPolicy policy, string destination, bool includeReferencedVocabularies)
        {
            Helper.BtsPolicyHelper.Export(BtsPolicy.GetQualifiedName(policy), destination, includeReferencedVocabularies);
        }

        public BtsPolicy ImportPolicy(string source)
        {
            RuleSetInfoCollection ruleSets;
            VocabularyInfoCollection vocabularies;

            Helper.BtsPolicyHelper.ImportAndPublish(source, out ruleSets, out vocabularies);

            // associate imported rulesets to the current application

            BtsResource resource = null;

            foreach (RuleSetInfo ruleSet in ruleSets)
            {
                BtsResource res = AddNewResource(
                    String.Format("RULE/{0}/{1}.{2}", ruleSet.Name, ruleSet.MajorRevision, ruleSet.MinorRevision)
                    , "System.BizTalk:Rules"
                    , true);

                resource = resource ?? res;
            }

            // only the first RuleSet is returned

            if (resource != null)
                return new BtsPolicy(this, resource);
            else
                return null;
        }

        public BtsPolicyCollection Policies
        {
            get { return new BtsPolicyCollection(this); }
        }

        public void RemovePolicy(BtsPolicy policy)
        {
            // first remove the policy from this application
            
            RemoveResource(policy.Resource);

            // then, remove the policy itself

            RuleSetInfo ruleSetInfo = new RuleSetInfo(policy.Name
                , policy.MajorRevision
                , policy.MinorRevision);

            if (Helper.BtsPolicyHelper.IsDeployed(ruleSetInfo))
                Helper.BtsPolicyHelper.Undeploy(ruleSetInfo);

            Helper.BtsPolicyHelper.Remove(ruleSetInfo);
        }

        #endregion

        #region Receive Locations

        public BtsReceiveLocationCollection ReceiveLocations
        {
            get { return new BtsReceiveLocationCollection(this); }
        }

        #endregion

        #region Receive Ports

        public void RemoveReceivePort(string name)
        {
            Catalog.RemoveReceivePort(name);
        }

        public void RemoveReceivePort(BtsReceivePort receivePort)
        {
            Catalog.RemoveReceivePort(receivePort);
        }

        public BtsReceivePortCollection ReceivePorts
        {
            get { return new BtsReceivePortCollection(this); }
        }

        #endregion

        #region Resources

        public BtsResource AddNewResource(string path)
        {
            return AddNewResource(path, false);
        }

        public BtsResource AddNewResource(string path, bool overwrite)
        {
            return AddNewResource(path, overwrite, false, false, false);
        }

        public BtsResource AddNewResource(string path, bool overwrite, bool gacOnAdd, bool gacOnInstall, bool gacOnImport)
        {
            string type = "System.BizTalk:File";
            string ext = Path.GetExtension(path).ToLower(System.Globalization.CultureInfo.InvariantCulture);

            if (ext == ".dll" || ext == ".ocx")
            {
                if (Helper.AppDomainHelper.IsBizTalkAssembly(path))
                    type = "System.BizTalk:BizTalkAssembly";
                else if (Helper.AppDomainHelper.IsManagedAssembly(path))
                    type = "System.BizTalk:Assembly";
            }

            if (ext == ".com" || ext == ".dll" || ext == ".exe")
            {
                if (IsComComponent(path))
                    type = "System.BizTalk:Com";
            }

            return AddNewResource(path, type, overwrite, gacOnAdd, gacOnInstall, gacOnImport);
        }
        
        public BtsResource AddNewResource(string path, string type, bool overwrite)
        {
            return AddNewResource(path, type, overwrite, false, false, false);
        }
        
        public BtsResource AddNewResource(string path, string type, bool overwrite, bool gacOnAdd, bool gacOnInstall, bool gacOnImport)
        {
            // store resource luid in order to be able to subsequently retrieve it

            string luid = String.Empty;

            // add resource - special case for System.BizTalk:Rules policies

            Dictionary<string, object> properties = new Dictionary<string, object>();
            Dictionary<string, string> idValues = new Dictionary<string, string>();

            string source = String.Empty;
            string destination = String.Empty;

            // special case for rules

            if (type == "System.BizTalk:Rules")
            {
                RuleSetInfo ruleSetInfo = Helper.BtsPolicyHelper.ParseResourceLuid(path);
                idValues.Add("Name", ruleSetInfo.Name);
                idValues.Add("Version", String.Format("{0}.{1}", ruleSetInfo.MajorRevision, ruleSetInfo.MinorRevision));
            }

            else if (type == "System.BizTalk:WebDirectory")
            {
                source = path;
                destination = path;
            }

            else
            {
                source = path;
                destination = Path.Combine("%BTAD_InstallDir%", Path.GetFileName(path));
            }

            // special case for bindings

            if (type == "System.BizTalk:Binding")
                properties.Add("TargetEnvironment", String.Empty);

            // populate appropriate properties

            if (type == "System.BizTalk:Assembly")
            {
                properties.Add("GacOnAdd", gacOnAdd);
                properties.Add("GacOnInstall", gacOnInstall);
                properties.Add("GacOnImport", gacOnImport);
                properties.Add("RegasmOnInstall", false);
                properties.Add("RegSvcsOnInstall", false);
            }

            else if (type == "System.BizTalk:BizTalkAssembly")
            {
                properties.Add("GacOnAdd", gacOnAdd);
                properties.Add("GacOnInstall", gacOnInstall);
                properties.Add("GacOnImport", gacOnImport);
            }

            else if (type == "System.BizTalk:Com")
            {
                properties.Add("Regsvr32OnInstall", false);
            }

            else if ((type == "System.BizTalk:PostProcessingScript") || (type == "System.BizTalk:PreProcessingScript"))
            {
                properties.Add("Args", String.Empty);
            }

            using (Microsoft.BizTalk.ApplicationDeployment.Group deploymentGroup = new Microsoft.BizTalk.ApplicationDeployment.Group())
            {
                deploymentGroup.DBName = Catalog.Database;
                deploymentGroup.DBServer = Catalog.Instance;

                try
                {
                    Microsoft.BizTalk.ApplicationDeployment.Application app = deploymentGroup.Applications[Name];
                    Microsoft.BizTalk.ApplicationDeployment.IDeploymentResource resourceOM;

                    if (type == "System.BizTalk:Rules")
                    {
                        resourceOM = app.ResourceCollection.Add(type, idValues, source, destination, properties, overwrite);
                    }
                    else
                    {
                        resourceOM = app.ResourceCollection.Add(type, source, destination, properties, overwrite);
                    }

                    deploymentGroup.Commit();

                    luid = resourceOM.Luid;
                }
                catch (System.Exception e)
                {
                    deploymentGroup.Abort();
                    throw new BtsException(e);
                }
            }

            // operations performed via ApplicationDeployment are committed in the BizTalk database.
            // force a Refresh() to keep Microsoft.BizTalk.ExplorerOM in sync.
            // Warning, this operation invalidates all objects currently in memory
            // including... this very BtsApplication object.

            string name = Name;
            Catalog.Refresh();
            application_ = Catalog.BtsCatalogExplorer.Applications[name];

            // retrieve newly added resource

            return Resources[luid];
        }

        public void RemoveResource(string type, string luid)
        {
            using (Microsoft.BizTalk.ApplicationDeployment.Group group = new Microsoft.BizTalk.ApplicationDeployment.Group())
            {
                group.DBName = Catalog.Database;
                group.DBServer = Catalog.Instance;

                try
                {

                    Microsoft.BizTalk.ApplicationDeployment.Application app = group.Applications[Name];
                    app.ResourceCollection.Remove(type, luid);
                }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }

            // operations performed via ApplicationDeployment are committed in the BizTalk database.
            // force a Refresh() to keep Microsoft.BizTalk.ExplorerOM in sync.
            // Warning, this operation invalidates all objects currently in memory
            // including... this very BtsApplication object.

            string name = Name;
            Catalog.Refresh();
            application_ = Catalog.BtsCatalogExplorer.Applications[name];
        }

        public void RemoveResource(BtsResource resource)
        {
            RemoveResource(resource.ResourceType, resource.Luid);
        }

        public void RemoveResources()
        {
            using (Microsoft.BizTalk.ApplicationDeployment.Group group = new Microsoft.BizTalk.ApplicationDeployment.Group())
            {
                group.DBName = Catalog.Database;
                group.DBServer = Catalog.Instance;

                try
                {
                    Microsoft.BizTalk.ApplicationDeployment.Application app = group.Applications[Name];
                    app.RemoveResources(app.ResourceCollection);
                }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }

            // operations performed via ApplicationDeployment are committed in the BizTalk database.
            // force a Refresh() to keep Microsoft.BizTalk.ExplorerOM in sync.
            // Warning, this operation invalidates all objects currently in memory
            // including... this very BtsApplication object.

            string name = Name;
            Catalog.Refresh();
            application_ = Catalog.BtsCatalogExplorer.Applications[name];
        }

        public BtsResourceCollection Resources
        {
            get { return new BtsResourceCollection(this); }
        }

        #endregion

        #region Schemas

        public void RemoveSchema(string name)
        {
            RemoveSchema(Schemas[name]);
        }

        public void RemoveSchema(BtsSchema schema)
        {
            RemoveResource("System.BizTalk:BizTalkAssembly", schema.AssemblyQualifiedName);
        }

        public BtsSchemaCollection Schemas
        {
            get { return new BtsSchemaCollection(this); }
        }

        #endregion

        #region Send Ports

        public void RemoveSendPort(string name)
        {
            Catalog.RemoveSendPort(name);
        }

        public void RemoveSendPort(BtsSendPort sendPort)
        {
            Catalog.RemoveSendPort(sendPort);
        }

        public BtsSendPortCollection SendPorts
        {
            get { return new BtsSendPortCollection(this); }
        }

        #endregion

        #region Send Port Groups

        public void RemoveSendPortGroup(string name)
        {
            Catalog.RemoveSendPortGroup(name);
        }

        public void RemoveSendPortGroup(BtsSendPortGroup sendPortGroup)
        {
            Catalog.RemoveSendPortGroup(sendPortGroup);
        }

        public BtsSendPortGroupCollection SendPortGroups
        {
            get { return new BtsSendPortGroupCollection(this); }
        }

        #endregion

        #region Export Application MSI

        public void Export(string package, XmlDocument resourceSpec, bool includeGlobalPartyBinding)
        {
            Catalog.ExportApplication(this, package, resourceSpec, includeGlobalPartyBinding);
        }

        public void Export(string package, XmlDocument resourceSpec)
        {
            Export(package, resourceSpec, true);
        }

        public void Export(string package, string resourceSpec, bool includeGlobalPartyBinding)
        {
            Catalog.ExportApplication(this, package, resourceSpec, includeGlobalPartyBinding);
        }

        public void Export(string package, string resourceSpec)
        {
            Export(package, resourceSpec, true);
        }

        public void Export(string package, bool includeGlobalPartyBinding)
        {
            Export(package, String.Empty, includeGlobalPartyBinding);
        }

        public void Export(string package)
        {
            Export(package, true);
        }

        #endregion

        #region AddNewResource Implementation Helpers

        [DllImport("kernel32", SetLastError = true)]
        private static extern IntPtr LoadLibrary(string path);
        [DllImport("kernel32", SetLastError = true)]
        private static extern bool FreeLibrary(IntPtr hModule);
        [DllImport("kernel32", CharSet = CharSet.Ansi, ExactSpelling = true, SetLastError = true)]
        private static extern UIntPtr GetProcAddress(IntPtr hModule, string name);

        private static bool IsComComponent(string path)
        {
            IntPtr hLibrary = LoadLibrary(path);
            if (hLibrary == IntPtr.Zero)
                return false;

            try
            {
                return (GetProcAddress(hLibrary, "DllGetClassObject") != UIntPtr.Zero);
            }
            finally
            {
                FreeLibrary(hLibrary);
            }
        }

        #endregion
    }
}
