using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.IO;
using System.Net;
using System.Reflection;
using System.Xml;
using BizTalkFactory.Management.Automation.Helper;
using Microsoft.BizTalk.Management;
using Microsoft.BizTalk.ExplorerOM;
using Microsoft.BizTalk.Deployment.Binding;
using Microsoft.BizTalk.Deployment;
using Microsoft.Win32;

namespace BizTalkFactory.Management.Automation
{
    public class BtsCatalog
    {
        private string database_ = "BizTalkMgmtDb";
        private string instance_ = Environment.MachineName;

        private GroupSetting setting_ = null;

        internal Microsoft.BizTalk.ExplorerOM.BtsCatalogExplorer explorer_ = new BtsCatalogExplorer();

        private BtsCatalog(string instance, string database)
        {
            database_ = database;
            instance_ = instance;

            // connect to the Microsoft.BizTalk.ExplorerOM.BtsCatalogExplorer

            BtsCatalogExplorer.ConnectionString = ConnectionString;

            RetrieveGroupSettings();
            RetrieveRuleEngineDbSettings();
        }

        public BtsCatalogExplorer BtsCatalogExplorer
        {
            get { return explorer_; }
        }

        public static BtsCatalog Connect(string instance, string database)
        {
            try
            {
                System.Diagnostics.Debugger.Break();
                return new BtsCatalog(instance, database);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public static BtsCatalog Connect(string server)
        {
            return Connect(server, "BizTalkMgmtDb");
        }

        private string ConnectionString
        {
            get { return String.Format("Data Source={0};Initial Catalog={1};Integrated Security=SSPI;", instance_, database_); }
        }

        public string Database
        {
            get { return database_; }
        }

        public string Name
        {
            get { return BtsCatalogExplorer.GroupName; }
            set
            {
                try
                {
                    setting_.Name = value;
                    Refresh();
                }

                catch (BtsException /* e */) { throw; }
                catch (System.Exception e) { throw new BtsException(e); }
            }
        }

        public string Instance
        {
            get { return instance_; }
        }

        #region Attributes

        public UInt32 ConfigurationCacheRefreshInterval
        {
            get { return setting_.ConfigurationCacheRefreshInterval; }
            set
            {
                try { setting_.ConfigurationCacheRefreshInterval = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public Boolean GlobalTrackingOption
        {
            get { return setting_.GlobalTrackingOption == GroupSetting.GlobalTrackingOptionValues.Normal_tracking; }
            set
            {
                try { setting_.GlobalTrackingOption = (value ? GroupSetting.GlobalTrackingOptionValues.Normal_tracking : GroupSetting.GlobalTrackingOptionValues.Disable_tracking); }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 LMSFragmentSize
        {
            get { return setting_.LMSFragmentSize; }
            set
            {
                try { setting_.LMSFragmentSize = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 LMSThreshold
        {
            get { return setting_.LMSThreshold; }
            set
            {
                try { setting_.LMSThreshold = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 PerfCounterCacheRefreshInterval
        {
            get { return setting_.PerfCounterCacheRefreshInterval; }
            set
            {
                try { setting_.PerfCounterCacheRefreshInterval = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        #endregion

        #region Adapters

        public BtsAdapter AddNewAdapter(string name, string transportType, string description)
        {
            var mgmtId = String.Empty;
            var constraints = 0U;

            if (!FindAdapterClassId(transportType, out mgmtId, out constraints))
                throw new BtsException(String.Format(BizTalkFactory.Management.Automation.Resources.BizTalkMgmt_AdapterNotFound, transportType));

            try
            {
                AdapterSetting.StaticScope = ManagementScopeHelper.GetScope(typeof (AdapterSetting), instance_, database_);
                AdapterSetting adapter = AdapterSetting.CreateInstance();

                adapter.LateBoundObject["Caption"] = name;
                adapter.LateBoundObject["Name"] = transportType;
                adapter.LateBoundObject["Comment"] = description;
                adapter.LateBoundObject["MgmtCLSID"] = mgmtId;
                adapter.LateBoundObject["Constraints"] = constraints;

                adapter.AutoCommit = true;
                adapter.CommitObject();

                // operations on WMI are carried out directly on the BizTalk database
                // force a refresh - to keep Microsoft.BizTalk.ExplorerOM in sync.

                Refresh();

                // return newly registered adapter

                return BtsAdapter.FromItem(this, adapter);

            }
            catch (System.Exception e)
            {
                throw new BtsException(e);
            }
            finally
            {
                AdapterSetting.StaticScope = null;
            }
        }

        private static bool FindAdapterClassId(string transportType, out string mgmtId, out uint constraints)
        {
            var CATID_BizTalkAdapter = new Guid("7F46FC3E-3C2C-405B-A47F-8D17942BA8F9");

            var classIds = Runtime.InteropServices.ComComponent.EnumImplementingClasses(CATID_BizTalkAdapter);
            foreach (var classId in classIds)
            {
                using (var regKey = Microsoft.Win32.Registry.ClassesRoot.OpenSubKey(String.Format(@"CLSID\{0:B}\BizTalk", classId)))
                {
                    if (transportType != (string) regKey.GetValue("TransportType")) continue;
                    mgmtId = classId.ToString("B").ToUpperInvariant();
                    constraints = Convert.ToUInt32((int) regKey.GetValue("Constraints"));
                    return true;
                }
            }

            mgmtId = string.Empty;
            constraints = 0;
            return false;
        }

        public void RemoveAdapter(string name)
        {
            try
            {
                AdapterSetting.StaticScope = ManagementScopeHelper.GetScope(typeof(AdapterSetting), instance_, database_);
                foreach (AdapterSetting adapter in AdapterSetting.GetInstances())
                    if (adapter.Name == name)
                        adapter.Delete();

                Refresh();
            }

            catch (System.Exception e)
            { throw new BtsException(e); }

            finally
            {
                AdapterSetting.StaticScope = null;
            }
        }

        public void RemoveAdapter(BtsAdapter adapter)
        {
            RemoveAdapter(adapter.Name);
        }

        #endregion

        #region Applications

        public BtsApplication AddNewApplication(string name, string description, bool isDefault, BtsApplication[] references)
        {
            try
            {
                Application application = BtsCatalogExplorer.AddNewApplication();

                application.Name = name;
                application.Description = description;
                foreach (BtsApplication reference in references)
                    application.AddReference(BtsCatalogExplorer.Applications[reference.Name]);

                if (isDefault)
                    BtsCatalogExplorer.DefaultApplication = application;

                SaveChanges();

                return BtsApplication.FromItem(this, application);
            }

            catch (BtsException /* e */) { throw; }
            catch (System.Exception e) { throw new BtsException(e); }
        }

        public BtsApplication AddNewApplication(string name, BtsApplication[] references)
        {
            return AddNewApplication(name, String.Empty, false, references);
        }

        public BtsApplication AddNewApplication(string name)
        {
            return AddNewApplication(name, new BtsApplication[] { });
        }

        public void ExportApplication(BtsApplication application, string package, string resourceSpec, bool includeGlobalPartyBinding)
        {
            Microsoft.BizTalk.ApplicationDeployment.ResourceSpec spec = new Microsoft.BizTalk.ApplicationDeployment.ResourceSpec();

            if (String.IsNullOrEmpty(resourceSpec))
            {
                XmlDocument document = Helper.BtsApplicationHelper.GetResourceSpec(application);
                resourceSpec = Path.GetTempFileName();
                using (System.IO.StreamWriter writer = new StreamWriter(File.OpenWrite(resourceSpec), System.Text.Encoding.Unicode))
                    writer.Write(document.OuterXml);
            }
            spec = Microsoft.BizTalk.ApplicationDeployment.ResourceSpec.LoadXml(resourceSpec);

            // export application with specified resource specs

            try
            {
                using (Microsoft.BizTalk.ApplicationDeployment.Group deploymentGroup = new Microsoft.BizTalk.ApplicationDeployment.Group())
                {
                    deploymentGroup.DBName = Database;
                    deploymentGroup.DBServer = Instance;

                    try
                    {
                        using (Microsoft.BizTalk.ApplicationDeployment.Application deploymentApp = deploymentGroup.Applications[application.Name])
                        {
                            if (deploymentApp.IsSystem)
                                throw new BtsException(BizTalkFactory.Management.Automation.Resources.BitzTalkMgmt_CannotExportSystemResources);

                            // retrieve resource luids

                            ICollection<String> luids = new List<String>();
                            foreach (Microsoft.BizTalk.ApplicationDeployment.ResourceSpecItem resource in spec.Resources)
                                luids.Add(resource.Luid);

                            // retrieve source locations

                            IDictionary<String, String> dictionary = new Dictionary<String, String>();
                            foreach (Microsoft.BizTalk.ApplicationDeployment.ResourceSpecItem resource in spec.Resources)
                            {
                                if (String.Compare(resource.Type, "System.BizTalk:WebDirectory", StringComparison.OrdinalIgnoreCase) == 0)
                                    dictionary.Add(resource.Luid, resource.Source);
                            }

                            // create resource list

                            ICollection<Microsoft.BizTalk.ApplicationDeployment.Resource> resources = new List<Microsoft.BizTalk.ApplicationDeployment.Resource>();
                            foreach (Microsoft.BizTalk.ApplicationDeployment.Resource resource in deploymentApp.ResourceCollection)
                            {
                                if (!luids.Contains(resource.Luid))
                                    continue;

                                if (resource.Properties.ContainsKey("IsSystem") && (String.Compare(bool.TrueString, resource.Properties["IsSystem"].ToString(), StringComparison.OrdinalIgnoreCase) == 0))
                                    // Cannot export system resource
                                    ;

                                else
                                {
                                    if (dictionary.ContainsKey(resource.Luid))
                                        resource.Properties["SourceLocation"] = dictionary[resource.Luid];

                                    if (includeGlobalPartyBinding && (string.Compare("Application/" + deploymentApp.Name, resource.Luid, StringComparison.Ordinal) == 0))
                                        resource.Properties["IncludeGlobalPartyBinding"] = includeGlobalPartyBinding;

                                    resources.Add(resource);
                                }
                            }

                            deploymentApp.Export(package, resources, null);
                        }
                    }
                    catch (System.Exception /* e */)
                    {
                        deploymentGroup.Abort();
                        throw;
                    }
                }
            }

            catch (BtsException /* e */) { throw; }
            catch (System.Exception e) { throw new BtsException(e); }
        }

        public void ExportApplication(BtsApplication application, string package, XmlDocument resourceSpec, bool includeGlobalPartyBinding)
        {
            // save XmlDocument to temporary file
            string tempfile = Path.GetTempFileName();
            resourceSpec.Save(tempfile);
            // export application
            ExportApplication(application, package, tempfile, true);
        }

        public void ExportApplication(BtsApplication application, string package, XmlDocument resourceSpec)
        {
            ExportApplication(application, package, resourceSpec, true);
        }

        public void ExportApplication(BtsApplication application, string package, bool includeGlobalPartyBinding)
        {
            ExportApplication(application, package, String.Empty, includeGlobalPartyBinding);
        }

        public void ExportApplication(BtsApplication application, string package, string resourceSpec)
        {
            ExportApplication(application, package, resourceSpec, true);
        }

        public void ExportApplication(BtsApplication application, string package)
        {
            ExportApplication(application, package, true);
        }

        public void ExportApplication(string name, string package, string resourceSpec, bool includeGlobalPartyBinding)
        {
            ExportApplication(Applications[name], package, resourceSpec, includeGlobalPartyBinding);
        }

        public void ExportApplication(string name, string package, string resourceSpec)
        {
            ExportApplication(name, package, resourceSpec, true);
        }

        public void ExportApplication(string name, string package, bool includeGlobalPartyBinding)
        {
            ExportApplication(Applications[name], package, includeGlobalPartyBinding);
        }

        public void ExportApplication(string name, string package)
        {
            ExportApplication(name, package, true);
        }

        public void ImportApplication(string name, string package, bool overwrite)
        {
            ImportApplication(name, package, String.Empty, overwrite);
        }

        public void ImportApplication(string name, string package, string stagingEnvironment, bool overwrite)
        {
            try
            {
                Microsoft.BizTalk.ApplicationDeployment.IInstallPackage msi = Microsoft.BizTalk.ApplicationDeployment.DeploymentUnit.ScanPackage(package);
                ICollection<string> applicationReferences = msi.References;

                if (String.IsNullOrEmpty(name))
                    name = msi.Title;

                using (Microsoft.BizTalk.ApplicationDeployment.Group deploymentGroup = new Microsoft.BizTalk.ApplicationDeployment.Group())
                {
                    deploymentGroup.DBName = Database;
                    deploymentGroup.DBServer = Instance;

                    try
                    {
                        using (Microsoft.BizTalk.ApplicationDeployment.Application deploymentApp =
                              (Microsoft.BizTalk.ApplicationDeployment.Application)deploymentGroup.CreateInstance(
                                typeof(Microsoft.BizTalk.ApplicationDeployment.Application)))
                        {

                            IDictionary<string, object> requestProperties = new Dictionary<string, object>();
                            if (!String.IsNullOrEmpty(stagingEnvironment) && (stagingEnvironment.Trim().Length > 0))
                                requestProperties.Add("TargetEnvironment", stagingEnvironment);

                            deploymentApp.UILevel = 2; // this statement is *required* !?
                            deploymentApp.Name = msi.Title;
                            deploymentApp.Import(package, requestProperties, applicationReferences, overwrite);
                        }
                    }
                    catch (System.Exception /* e */)
                    {
                        deploymentGroup.Abort();
                        throw;
                    }
                }
            }

            catch (System.Exception e)
            { throw new BtsException(e); }

            // operations on the Microsoft.BizTalk.ApplicationDeployment.Engine are carried out directly on the BizTalk database
            // force a refresh - to keep Microsoft.BizTalk.ExplorerOM in sync.

            Refresh();
        }

        public void ImportApplication(string package)
        {
            ImportApplication(String.Empty, package, "ALL", true);
        }

        public void RemoveApplication(string name)
        {
            RemoveApplication(name, false);
        }

        public void RemoveApplication(BtsApplication application)
        {
            RemoveApplication(application, false);
        }

        public void RemoveApplication(string name, bool recursive)
        {
            RemoveApplication(Applications[name], recursive);
        }

        public void RemoveApplication(BtsApplication application, bool recursive)
        {
            try
            {
                application.Stop(BtsApplicationStopOption.StopAll & ~BtsApplicationStopOption.StopReferencedApplications);
                SaveChanges();

                if (recursive)
                {
                    Helper.BtsApplicationHelper.ResetPipelines(application);

                    // operations performed on resources force a Catalog Refresh
                    // this invalidates all objects currently in memory...
                    // including the BtsApplication object supplied as arguments
                    // to this method.

                    string name = application.Name;
                    application.RemoveResources();
                    application = application.Catalog.Applications[name];

                    // proceed with remaining items

                    Helper.BtsApplicationHelper.RemoveAllReceivePorts(application);
                    Helper.BtsApplicationHelper.RemoveAllSendPortGroups(application);
                    Helper.BtsApplicationHelper.RemoveAllSendPorts(application);
                }

                BtsCatalogExplorer.RemoveApplication(BtsCatalogExplorer.Applications[application.Name]);
                SaveChanges();
            }

            catch (BtsException /* e */) { throw; }
            catch (System.Exception e) { throw new BtsException(e); }
        }

        public void SetDefaultApplication(string name)
        {
            try
            {
                BtsCatalogExplorer.DefaultApplication = BtsCatalogExplorer.Applications[name];
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public void SetDefaultApplication(BtsApplication application)
        {
            SetDefaultApplication(application.Name);
        }

        public BtsApplicationCollection Applications
        {
            get { return new BtsApplicationCollection(this); }
        }

        #endregion

        #region Bindings

        public void ExportBinding(string destination)
        {
            ExportBinding(this, null, destination, false);
        }

        public void ExportBinding(BtsApplication application, string destination, bool includeGlobalPartyBinding)
        {
            ExportBinding(this, application, destination, includeGlobalPartyBinding);
        }

        public void ExportBinding(BtsResource resource, string destination, bool includeGlobalPartyBinding)
        {
            ExportBinding(this, resource, destination, includeGlobalPartyBinding);
        }

        private static void ExportBinding(BtsCatalog catalog, BtsArtifact item, string destination, bool includeGlobalPartyBinding)
        {
            try
            {
                bool group = false;

                using (SqlConnection sqlConnection = new SqlConnection(catalog.BtsCatalogExplorer.ConnectionString))
                {
                    // apply binding parameters 

                    BindingInfo info = new BindingInfo();
                    BindingParameters bindingParameters = new BindingParameters(new Version(info.Version));
                    bindingParameters.BindingItems = BindingParameters.BindingItemTypes.All;

                    if (item == null)
                    {
                        group = true;
                        bindingParameters.BindingScope = BindingParameters.BindingScopeType.Group;
                    }

                    //else if (item is BtsAssembly)
                    //{
                    //    BtsAssembly assembly = (item as BtsAssembly);

                    //    bindingParameters.BindingScope = BindingParameters.BindingScopeType.None;
                    //    AssemblyName name = new AssemblyName(assembly.DisplayName);
                    //    string culture = (((name.CultureInfo != null) && (name.CultureInfo.Name != null)) && (name.CultureInfo.Name.Length > 0)) ? name.CultureInfo.Name : "neutral";
                    //    string token = ByteArrayToHexString(name.GetPublicKeyToken());
                    //    info.AddModuleRef(name.Name, name.Version.ToString(), culture, token);
                    //}

                    else if (item is BtsResource)
                    {
                        BtsResource resource = (item as BtsResource);

                        if (resource.ResourceType != "System.BizTalk:Assembly" && resource.ResourceType != "System.BizTalk:BizTalkAssembly")
                            throw new BtsException(String.Format(
                                BizTalkFactory.Management.Automation.Resources.BizTalkMgmt_InvalidResourceType
                                , resource.ResourceType));

                        bindingParameters.BindingScope = BindingParameters.BindingScopeType.None;
                        AssemblyName name = new AssemblyName(resource.DisplayName);
                        string culture = (((name.CultureInfo != null) && (name.CultureInfo.Name != null)) && (name.CultureInfo.Name.Length > 0)) ? name.CultureInfo.Name : "neutral";
                        string token = ByteArrayToHexString(name.GetPublicKeyToken());
                        info.AddModuleRef(name.Name, name.Version.ToString(), culture, token);
                    }

                    else if (item is BtsApplication)
                    {
                        BtsApplication application = (item as BtsApplication);

                        bindingParameters.BindingScope = BindingParameters.BindingScopeType.Application;
                        info.AddApplicationRef(sqlConnection, application.Name);
                    }

                    if (includeGlobalPartyBinding && !group) // not valid for group level
                        bindingParameters.IncludeGlobalParty = true;

                    // export bindings

                    info.Select(sqlConnection, bindingParameters);
                    destination = System.IO.Path.GetFullPath(destination);
                    string directoryName = System.IO.Path.GetDirectoryName(destination);
                    if (!Directory.Exists(directoryName))
                        Directory.CreateDirectory(directoryName);
                    info.SaveXml(destination);
                }
            }

            catch (BtsException /* e */) { throw; }
            catch (System.Exception e) { throw new BtsException(e); }
        }

        public void ImportBinding(BtsApplication application, string source)
        {
            ImportBinding(BtsCatalogExplorer.ConnectionString, application.Name, source);
        }

        public void ImportBinding(string source)
        {
            ImportBinding(BtsCatalogExplorer.ConnectionString, null, source);
        }

        private void ImportBinding(string connectionstring, string applicationName, string source)
        {
            string errorMessage = String.Empty;

            try
            {
                using (DeployerComponent deployerComponent = new DeployerComponent())
                {
                    switch (deployerComponent.ImportBindingWithValidation(connectionstring, source, applicationName, false, ref errorMessage))
                    {
                        case ImportBindingError.Failed:
                            throw new BtsException(errorMessage);

                        case ImportBindingError.SucceededWithWarning:
                        case ImportBindingError.Succeeded:
                            break;
                    }
                }
            }

            catch (BtsException /* e */) { throw; }
            catch (System.Exception e) { throw new BtsException(e); }
        }

        private static string ByteArrayToHexString(byte[] bytes)
        {
            System.Text.StringBuilder builder = new System.Text.StringBuilder();
            if (bytes != null)
            {
                foreach (byte num in bytes)
                    builder.Append(num.ToString("x2", System.Globalization.CultureInfo.InvariantCulture));
            }
            return builder.ToString();
        }

        #endregion

        #region Group
#if BTS2010

        public void ExportSettings(string path)
        {
            try
            {
                SettingsWorker settingsWorker = new SettingsWorker(Instance, Database);
                settingsWorker.ExportAllSettings(path);
            }
            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public void ImportSettings(string path)
        {
            SettingsWorker settingsWorker = new SettingsWorker(Instance, Database);

            ImportGroupSettings(settingsWorker, path);
            ImportHostsSettings(settingsWorker, path);
            ImportHostInstancesSettings(settingsWorker, path);

            RetrieveGroupSettings();
        }

        public void ExportGroupSettings(string path)
        {
        }

        public void ExportHostsSettings(string path)
        {
        }

        public void ExportHostInstancesSettings(string path)
        {
        }

        public void ImportGroupSettings(string path)
        {
            SettingsWorker settingsWorker = new SettingsWorker(Instance, Database);
            ImportGroupSettings(settingsWorker, path);
            RetrieveGroupSettings();
        }

        private void ImportGroupSettings(SettingsWorker settingsWorker, string path)
        {
            SettingsRoot settingsRoot = Helper.BtsSettingsHelper.ParseGroupSettings(path);

            try
            {
                ExportedSettings exportedSettings = new ExportedSettings();
                exportedSettings.ExportedGroup = String.Format("{0}:{1}", Instance.ToUpper(), Database.ToUpper());
                exportedSettings.GroupSettings = settingsRoot;

                settingsWorker.ImportGroupSettings(exportedSettings);
            }

            catch (Exception e)
            {
                throw new BtsException(settingsWorker.GetManagementExceptionDescription(e), e);
            }
        }

        public void ImportHostsSettings(string path)
        {
            SettingsWorker settingsWorker = new SettingsWorker(Instance, Database);
            ImportHostsSettings(settingsWorker, path);
        }

        public void ImportHostsSettings(SettingsWorker settingsWorker, string path)
        {
            HostSettings hostSettings = Helper.BtsSettingsHelper.ParseHostsSettings(path);
            foreach (BtsHost host in Hosts)
                host.ImportHostSettings(settingsWorker, hostSettings);
        }

        public void ImportHostInstancesSettings(string path)
        {
            SettingsWorker settingsWorker = new SettingsWorker(Instance, Database);
            ImportHostInstancesSettings(settingsWorker, path);
        }

        private void ImportHostInstancesSettings(SettingsWorker settingsWorker, string path)
        {
            HostInstanceSettings hostInstanceSettings = Helper.BtsSettingsHelper.ParseHostInstancesSettings(path);
            foreach (BtsHostInstance hostInstance in HostInstances)
                hostInstance.ImportHostInstanceSettings(settingsWorker, hostInstanceSettings);
        }
#endif
        #endregion

        #region Hosts

        public BtsHost AddNewHost(string Name, BtsHostType HostType, string NtGroupName, bool AuthTrusted)
        {
            try
            {
                // create new BizTalk Host

                HostSetting.StaticScope = ManagementScopeHelper.GetScope(typeof(HostSetting), instance_, database_);
                HostSetting host = HostSetting.CreateInstance();
                host.AutoCommit = false;

                host.LateBoundObject["Name"] = Name;
                host.LateBoundObject["HostType"] = HostType;
                host.LateBoundObject["NTGroupName"] = NtGroupName;
                host.LateBoundObject["AuthTrusted"] = AuthTrusted;

                host.AutoCommit = true;
                host.CommitObject();
            }

            catch (System.Exception e)
            { throw new BtsException(e); }

            finally
            {
                HostSetting.StaticScope = null;
            }

            // operations performed via WMI are committed in the BizTalk database.
            // force a Refresh() to keep Microsoft.BizTalk.ExplorerOM in sync.

            Refresh();

            // retrieve object

            return BtsHost.FromItem(this, BtsCatalogExplorer.Hosts[Name]);
        }

        public void RemoveHost(string Name)
        {
            try
            {
                HostSetting.StaticScope = ManagementScopeHelper.GetScope(typeof(HostSetting), instance_, database_);
                foreach (HostSetting host in HostSetting.GetInstances())
                    if (host.Name == Name)
                        host.Delete();
            }

            catch (System.Exception e)
            { throw new BtsException(e); }

            finally
            {
                HostSetting.StaticScope = null;
            }

            // operations performed via WMI are committed in the BizTalk database.
            // force a Refresh() to keep Microsoft.BizTalk.ExplorerOM in sync.

            Refresh();
        }

        public void RemoveHost(BtsHost host)
        {
            RemoveHost(host.Name);
        }

        public BtsHostCollection Hosts
        {
            get { return new BtsHostCollection(this); }
        }

        #endregion

        #region Host Instances

        public BtsHostInstance AddNewHostInstance(string HostName)
        {
            return AddNewHostInstance(HostName, Environment.MachineName);
        }

        public BtsHostInstance AddNewHostInstance(string HostName, string RunningServer)
        {
            return AddNewHostInstance(HostName, RunningServer, CredentialCache.DefaultNetworkCredentials);
        }

        public BtsHostInstance AddNewHostInstance(string HostName, string RunningServer, NetworkCredential Credential)
        {
            try
            {
                // map BizTalk Host

                ServerHost.StaticScope = ManagementScopeHelper.GetScope(typeof(ServerHost), instance_, database_);
                ServerHost host = ServerHost.CreateInstance();
                host.LateBoundObject["ServerName"] = RunningServer;
                host.LateBoundObject["HostName"] = HostName;
                host.Map();

                // add new BizTalk Host Instance

                string name = BtsHostInstance.GetNameFromHostName(HostName, RunningServer);
                HostInstance.StaticScope = ManagementScopeHelper.GetScope(typeof(HostInstance), instance_, database_);
                HostInstance hostInstance = HostInstance.CreateInstance();
                hostInstance.LateBoundObject["Name"] = name;

                if (!String.IsNullOrEmpty(Credential.Domain))
                    hostInstance.Install(true, String.Format(@"{0}\{1}", Credential.Domain, Credential.UserName), Credential.Password);
                else
                    hostInstance.Install(true, Credential.UserName, Credential.Password);

                // retrieve object

                return HostInstances[name];
            }

            catch (System.Exception e)
            { throw new BtsException(e); }

            finally
            {
                ServerHost.StaticScope = null;
                HostInstance.StaticScope = null;
            }
        }

        public void RemoveHostInstance(string HostName, string RunningServer)
        {
            string name = BtsHostInstance.GetNameFromHostName(HostName, RunningServer);

            try
            {
                // retrieve underlying HostInstance Management object

                HostInstance instance = null;
                HostInstance.StaticScope = ManagementScopeHelper.GetScope(typeof(HostSetting), instance_, database_);
                foreach (HostInstance currInstance in HostInstance.GetInstances())
                {
                    if (String.Compare(currInstance.Name, name, StringComparison.InvariantCultureIgnoreCase) == 0)
                    {
                        instance = currInstance;
                        break;
                    }
                }

                // uninstall host instance

                instance.Uninstall();

                // delete host instance

                ServerHost.StaticScope = ManagementScopeHelper.GetScope(typeof(HostInstance), instance_, database_);
                ServerHost host = ServerHost.CreateInstance();

                host.LateBoundObject["ServerName"] = RunningServer;
                host.LateBoundObject["HostName"] = HostName;
                host.Unmap();

            }

            catch (System.Exception e)
            { throw new BtsException(e); }

            finally
            {
                HostInstance.StaticScope = null;
                ServerHost.StaticScope = null;
            }
        }

        public void RemoveHostInstance(BtsHostInstance hostInstance)
        {
            RemoveHostInstance(hostInstance.HostName, hostInstance.RunningServer);
        }

        public BtsHostInstanceCollection HostInstances
        {
            get { return new BtsHostInstanceCollection(this); }
        }

        #endregion

        #region Orchestrations

        public BtsOrchestrationCollection Orchestrations
        {
            get { return new BtsOrchestrationCollection(this); }
        }

        #endregion

        #region Parties

        public void RemoveParty(BtsParty party)
        {
            try
            {
                BtsCatalogExplorer.RemoveParty(BtsCatalogExplorer.Parties[party.Name]);
                SaveChanges();
            }

            catch (BtsException /* e */) { throw; }
            catch (System.Exception e) { throw new BtsException(e); }
        }

        #endregion

        #region Pipelines

        public BtsPipelineCollection Pipelines
        {
            get { return new BtsPipelineCollection(this); }
        }

        #endregion

        #region Policies

        public BtsPolicyCollection Policies
        {
            get { return new BtsPolicyCollection(this); }
        }

        #endregion

        #region Receive Handlers

        public void RemoveHandler(BtsReceiveHandler handler)
        {
            try
            {
                // retrieve underlying ReceiveHandler Management object

                Microsoft.BizTalk.Management.ReceiveHandler receiveHandler = null;
                Microsoft.BizTalk.Management.ReceiveHandler.StaticScope =
                    ManagementScopeHelper.GetScope(
                        typeof(Microsoft.BizTalk.Management.ReceiveHandler), instance_, database_);

                foreach (Microsoft.BizTalk.Management.ReceiveHandler currHandler in Microsoft.BizTalk.Management.ReceiveHandler.GetInstances())
                {
                    if (String.Compare(currHandler.HostName, handler.HostName, StringComparison.InvariantCultureIgnoreCase) == 0 &&
                        String.Compare(currHandler.AdapterName, handler.AdapterName, StringComparison.InvariantCultureIgnoreCase) == 0)
                    {
                        receiveHandler = currHandler;
                        break;
                    }
                }


                // remove send handler

                receiveHandler.Delete();
            }

            catch (System.Exception e)
            { throw new BtsException(e); }

            finally
            {
                Microsoft.BizTalk.Management.ReceiveHandler.StaticScope = null;
            }
        }

        #endregion

        #region Receive Locations

        public BtsReceiveLocationCollection ReceiveLocations
        {
            get { return new BtsReceiveLocationCollection(this); }
        }

        #endregion

        #region Receive Ports

        public void RemoveReceivePort(string name)
        {
            try
            {
                BtsCatalogExplorer.RemoveReceivePort(BtsCatalogExplorer.ReceivePorts[name]);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public void RemoveReceivePort(BtsReceivePort receivePort)
        {
            RemoveReceivePort(receivePort.Name);
        }

        public BtsReceivePortCollection ReceivePorts
        {
            get { return new BtsReceivePortCollection(this); }
        }

        #endregion

        #region Resources

        public BtsResourceCollection Resources
        {
            get { return new BtsResourceCollection(this); }
        }

        #endregion

        #region Schemas

        public BtsSchemaCollection Schemas
        {
            get { return new BtsSchemaCollection(this); }
        }

        #endregion

        #region Send Handlers

        public void RemoveHandler(BtsSendHandler handler)
        {
            try
            {
                // retrieve underlying sendHandler2 Management object

                SendHandler2 sendHandler = null;
                SendHandler2.StaticScope = ManagementScopeHelper.GetScope(typeof(SendHandler2), instance_, database_);

                foreach (SendHandler2 currHandler in SendHandler2.GetInstances())
                {
                    if (String.Compare(currHandler.HostName, handler.HostName, StringComparison.InvariantCultureIgnoreCase) == 0 &&
                        String.Compare(currHandler.AdapterName, handler.AdapterName, StringComparison.InvariantCultureIgnoreCase) == 0)
                    {
                        sendHandler = currHandler;
                        break;
                    }
                }


                // remove send handler

                sendHandler.Delete();
            }

            catch (System.Exception e)
            { throw new BtsException(e); }

            finally
            {
                SendHandler2.StaticScope = null;
            }
        }

        #endregion

        #region Send Ports

        public void RemoveSendPort(string name)
        {
            try
            {
                BtsCatalogExplorer.RemoveSendPort(BtsCatalogExplorer.SendPorts[name]);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public void RemoveSendPort(BtsSendPort sendPort)
        {
            RemoveSendPort(sendPort.Name);
        }

        public BtsSendPortCollection SendPorts
        {
            get { return new BtsSendPortCollection(this); }
        }

        #endregion

        #region Send Port Groups

        public void RemoveSendPortGroup(string name)
        {
            try
            {
                BtsCatalogExplorer.RemoveSendPortGroup(BtsCatalogExplorer.SendPortGroups[name]);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public void RemoveSendPortGroup(BtsSendPortGroup sendPortGroup)
        {
            RemoveSendPortGroup(sendPortGroup.Name);
        }

        public BtsSendPortGroupCollection SendPortGroups
        {
            get { return new BtsSendPortGroupCollection(this); }
        }

        #endregion

        public void SaveChanges()
        {
            try
            {
                BtsCatalogExplorer.SaveChanges();
            }
            catch (System.Exception e)
            {
                throw new BtsException(BizTalkFactory.Management.Automation.Resources.BitzTalkMgmt_UnableToCommit, e);
            }
        }

        public void Refresh()
        {
            try
            {
                BtsCatalogExplorer.Refresh();
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        #region Implementation

        private void RetrieveGroupSettings()
        {
            // retrieve the first WMI MSBTS_GroupSetting object in the collection

            try
            {
                GroupSetting.StaticScope = ManagementScopeHelper.GetScope(typeof(GroupSetting), instance_, database_);
                foreach (GroupSetting setting in GroupSetting.GetInstances())
                {
                    setting_ = setting;
                    break;
                }
            }

            finally
            {
                GroupSetting.StaticScope = null;
            }
        }

        private void RetrieveRuleEngineDbSettings()
        {
            System.Diagnostics.Debugger.Break();

            const string statement = "SELECT RuleEngineDBName, RuleEngineDBServerName FROM [dbo].[adm_Group]";

            using (var connection = new System.Data.SqlClient.SqlConnection(explorer_.ConnectionString))
            using (var command = new System.Data.SqlClient.SqlCommand(statement, connection))
            {
                command.CommandType = System.Data.CommandType.Text;
                connection.Open();

                using (var reader = command.ExecuteReader(System.Data.CommandBehavior.SingleRow))
                {
                    if (!reader.Read())
                        return;

                    BtsPolicyHelper.BizTalkruleEngineRunningServer = reader["RuleEngineDBServerName"] as String;
                    BtsPolicyHelper.BizTalkRuleEngineDb = reader["RuleEngineDBName"] as String;
                }
            }
        }

        #endregion
    }
}
