using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;

namespace BizTalkFactory.Management.Automation
{
    public class BtsCollection<T> : IReadOnlyCollectionWrapper<T> where T : BtsArtifact
    {
        protected BtsCatalog catalog_;

        public BtsCollection(BtsCatalog catalog, ICollection collection)
            : base(collection)
        {
            catalog_ = catalog;
        }

        public BtsCatalog Catalog
        {
            get { return catalog_; }
        }

        public virtual T this[string name]
        {
            get { return GetItem(name); }
        }

        public virtual bool TryExists(string name)
        {
            try
            {
                T t = GetItem(name);
                return true;
            }
            catch (KeyNotFoundException /* e */)
            {
                return false;
            }
        }

        #region Support Helpers

        internal T GetItem(int index)
        {
            int position = 0;
            IEnumerator<T> enumerator = GetEnumerator();
            while (enumerator.MoveNext())
            {
                if (position++ == index)
                    return enumerator.Current;
            }

            throw new IndexOutOfRangeException();
        }

        internal T GetItem(string name)
        {
            IEnumerator<T> enumerator = GetEnumerator();
            while (enumerator.MoveNext())
            {
                if (String.Compare(enumerator.Current.Name, name, StringComparison.InvariantCultureIgnoreCase) == 0)
                    return enumerator.Current;
            }
            
            throw new KeyNotFoundException();
        }

        internal T FromItem(object item)
        {
            MethodInfo FromItemMethod = typeof(T).GetMethod("FromItem");
            return FromItemMethod.Invoke(null, new object[]{Catalog, item}) as T;
        }

        #endregion

        #region ICollection<T> Overrides

        public override bool Contains(T item)
        {
            foreach (object o_item in Collection)
            {
                T t_item = FromItem(o_item);
                if (t_item.Equals(item))
                    return true;
            }

            return false;
        }

        public override void CopyTo(T[] array, int index)
        {
            for (int n = index; (n < array.Length); n++)
                array.SetValue(GetItem(n - index), n);
        }

        #endregion

        #region IEnumerable<T> Overrides

        public override IEnumerator<T> GetEnumerator()
        {
            return new BtsEnumerator<T>(this);
        }

        #endregion
    }

    public sealed class BtsEnumerator<T> : IEnumeratorWrapper<T> where T : BtsArtifact
    {
        private BtsCollection<T> collection_;

        internal BtsEnumerator(BtsCollection<T> collection)
            : base(collection.Collection.GetEnumerator())
        {
            collection_ = collection;
        }

        #region IEnumerator<T> Overrides

        public override T Current
        {
            get
            {
                return collection_.FromItem(Item);
            }
        }

        #endregion
    }
}
