using System;
using System.Net;

using Microsoft.BizTalk.Management;
using Microsoft.BizTalk.ExplorerOM;

namespace BizTalkFactory.Management.Automation
{
    public class BtsHostInstance : BtsArtifact
    {
        private HostInstance instance_;
        private HostInstanceSetting setting_;

        private BtsHostInstance(BtsCatalog catalog, HostInstance instance)
            : base(catalog)
        {
            instance_ = instance;

            RetrieveHostInstanceSetting(instance.Name);
        }

        public static BtsHostInstance FromItem(BtsCatalog catalog, object item)
        {
            return new BtsHostInstance(catalog, item as HostInstance);
        }

        #region Attributes

        public bool IsDisabled
        {
            get { return instance_.IsDisabled; }
            set
            {
                try { instance_.IsDisabled = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public override string Name
        {
            get { return instance_.Name; }
        }

        public string HostName
        {
            get { return instance_.HostName; }
        }

        public string NtGroupName
        {
            get { return instance_.NTGroupName; }
        }

        public string NtUserName
        {
            get { return instance_.Logon; }
        }

        public string RunningServer
        {
            get { return instance_.RunningServer; }
        }

        public BtsHostType HostType
        {
            get { return HostTypeConverter.FromManagementType((HostSetting.HostTypeValues)instance_.HostType); }
        }

        public BtsServiceState ServiceState
        {
            get { return ServiceStateConverter.FromManagementType(instance_.ServiceState); }
        }

        public UInt32 CLRMaxIOThreads
        {
            get { return setting_.CLRMaxIOThreads; }
            set
            {
                try { setting_.CLRMaxIOThreads = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 CLRMaxWorkerThreads
        {
            get { return setting_.CLRMaxWorkerThreads; }
            set
            {
                try { setting_.CLRMaxWorkerThreads = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 CLRMinIOThreads
        {
            get { return setting_.CLRMinIOThreads; }
            set
            {
                try { setting_.CLRMinIOThreads = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 CLRMinWorkerThreads
        {
            get { return setting_.CLRMinWorkerThreads; }
            set
            {
                try { setting_.CLRMinWorkerThreads = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 PhysicalMemoryMaximalUsage
        {
            get { return setting_.PhysicalMemoryMaximalUsage; }
            set
            {
                try { setting_.PhysicalMemoryMaximalUsage = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 PhysicalMemoryOptimalUsage
        {
            get { return setting_.PhysicalMemoryOptimalUsage; }
            set
            {
                try { setting_.PhysicalMemoryOptimalUsage = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }


        public UInt32 VirtualMemoryMaximalUsage
        {
            get { return setting_.VirtualMemoryMaximalUsage; }
            set
            {
                try { setting_.VirtualMemoryMaximalUsage = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        public UInt32 VirtualMemoryOptimalUsage
        {
            get { return setting_.VirtualMemoryOptimalUsage; }
            set
            {
                try { setting_.VirtualMemoryOptimalUsage = value; }
                catch (System.Exception e)
                { throw new BtsException(e); }
            }
        }

        #endregion

        #region Operations

        public void Configure(NetworkCredential credential)
        {
            try
            {
                // will throw if not previously stopped (WinMgmt)

                instance_.Uninstall();
                instance_.Install(true, credential.UserName, credential.Password);
            }
            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public void Start()
        {
            if (HostType == BtsHostType.Isolated)
                return;

            try
            {
                instance_.Start();
            }
            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public void Stop()
        {
            if (HostType == BtsHostType.Isolated)
                return;

            try
            {
                instance_.Stop();
            }
            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public void Restart()
        {
            Stop();
            Start();
        }

        public static string GetNameFromHostName(string HostName, string RunningServer)
        {
            return "Microsoft BizTalk Server " + HostName + " " + RunningServer;
        }

#if BTS2010
        public void ImportHostInstanceSettings(string path)
        {
            SettingsWorker settingsWorker = new SettingsWorker(Catalog.Instance, Catalog.Database);
            HostInstanceSettings hostInstanceSettings = Helper.BtsSettingsHelper.ParseHostInstancesSettings(path);

            ImportHostInstanceSettings(settingsWorker, hostInstanceSettings);

            RetrieveHostInstanceSetting(Name);
        }

        
        public void ImportHostInstanceSettings(SettingsWorker settingsWorker, HostInstanceSettings settings)
        {
            ExportedSettings exportedSettings = new ExportedSettings();
            exportedSettings.ExportedGroup = String.Format("{0}:{1}", Catalog.Instance.ToUpper(), Catalog.Database.ToUpper());
            exportedSettings.HostInstanceSettings = settings;

            try
            {
                String hostInstanceName = String.Format("{0}:{1}", HostName, RunningServer);
                settingsWorker.ImportHostInstanceSettings(hostInstanceName, hostInstanceName, exportedSettings);
            }
            catch (System.Exception e)
            {
                throw new BtsException(settingsWorker.GetManagementExceptionDescription(e), e);
            }
        }
#endif

        #endregion

        #region Implementation

        private void RetrieveHostInstanceSetting(string name)
        {
            try
            {
                HostInstanceSetting.StaticScope = ManagementScopeHelper.GetScope(typeof(HostInstanceSetting), Catalog.Instance, Catalog.Database);
                foreach (HostInstanceSetting setting in HostInstanceSetting.GetInstances())
                    if (name == setting.Name)
                    {
                        setting_ = setting;
                        break;
                    }
            }

            finally
            {
                HostInstanceSetting.StaticScope = null;
            }
        }

        #endregion
    }
}
