using System;
using System.Collections.Generic;

using Microsoft.RuleEngine;

namespace BizTalkFactory.Management.Automation
{
    public class BtsPolicy : BtsApplicationArtifact
    {
        private BtsResource resource_;
        private RuleSetInfo ruleInfo_;

        internal BtsPolicy(BtsApplication application, BtsResource resource)
            : base(application)
        {
            System.Diagnostics.Debug.Assert(resource.ResourceType == "System.BizTalk:Rules");

            resource_ = resource;
            ruleInfo_ = Helper.BtsPolicyHelper.GetRuleSetInfoFromLuid(resource_.Luid);
        }

        public static BtsPolicy FromItem(BtsCatalog catalog, object item)
        {
            try
            {
                BtsResource resource = (item as BtsResource);
                BtsApplication application = BtsApplication.FromItem(catalog, resource.Application);
                return new BtsPolicy(application, resource);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        #region Attributes

        public bool Deployed
        {
            get { return Helper.BtsPolicyHelper.IsDeployed(ruleInfo_); }
        }

        public int MajorRevision
        {
            get { return ruleInfo_.MajorRevision; }
        }

        public int MinorRevision
        {
            get { return ruleInfo_.MinorRevision; }
        }

        public override string Name
        {
            get { return ruleInfo_.Name; }
        }

        public bool Published
        {
            get { return ruleInfo_.Published; }
        }

        internal BtsResource Resource
        {
            get { return resource_; }
        }

        #endregion

        #region Operations

        public void Deploy()
        {
            Helper.BtsPolicyHelper.Deploy(ruleInfo_);
        }

        public void Undeploy()
        {
            Helper.BtsPolicyHelper.Undeploy(ruleInfo_);
        }

        #endregion

        #region Helpers

        public static string GetVersion(BtsPolicy policy)
        {
            return String.Format("{0}.{1}", policy.MajorRevision, policy.MinorRevision);
        }

        public static string GetQualifiedName(BtsPolicy policy)
        {
            return String.Format("{0}:{1}", policy.Name, GetVersion(policy));
        }

        #endregion

        #region Implementation

        private RuleSetInfo UpdateRuleSetInfoVersion(RuleSetInfo ruleSetInfo, string version)
        {
            int majorRevision = ruleSetInfo.MajorRevision;
            int minorRevision = ruleSetInfo.MinorRevision;

            string[] components = version.Split('.');
            if (components.Length == 1)
            {
                majorRevision = Int32.Parse(components[0]);
                minorRevision = 0;
            }

            else if (components.Length == 2)
            {
                majorRevision = Int32.Parse(components[0]);
                minorRevision = Int32.Parse(components[1]);
            }

            return new RuleSetInfo(
                ruleSetInfo.Name
                , majorRevision
                , minorRevision
                , ruleSetInfo.Description
                , ruleSetInfo.ModifiedTime
                , ruleSetInfo.ModifiedBy
                , ruleSetInfo.Published);
        }

        #endregion

        #region IComparable<BtsPolicy> Members

        internal class BtsPolicyComparer : IComparer<BtsPolicy>
        {
            int IComparer<BtsPolicy>.Compare(BtsPolicy left, BtsPolicy right)
            {
                if (left.MajorRevision < right.MajorRevision) return -1;
                else if (left.MajorRevision > right.MinorRevision) return 1;
                else if (left.MinorRevision < right.MinorRevision) return -1;
                else if (left.MinorRevision > right.MinorRevision) return 1;
                else return 0;
            }
        }

        public static IComparer<BtsPolicy> Comparer
        {
            get { return new BtsPolicyComparer(); }
        }

        #endregion
    }
}
