﻿using System;
using System.IO;
using System.Xml;

using Microsoft.Win32;

using BizTalkFactory.Management.Automation;

namespace BizTalkFactory.Management.Automation.Helper
{
    public static class BtsApplicationHelper
    {
        public static BtsReceiveLocation FindReceiveLocation(BtsApplication application, string receiveLocationName)
        {
            BtsReceiveLocation receiveLocation = null;
            foreach (BtsReceivePort port in application.ReceivePorts)
                foreach (BtsReceiveLocation location in port.ReceiveLocations)
                    if (String.Compare(location.Name, receiveLocationName, StringComparison.InvariantCultureIgnoreCase) == 0)
                    {
                        receiveLocation = location;
                        break;
                    }

            return receiveLocation;
        }

        public static BtsApplication GetSystemApplication(BtsCatalog catalog)
        {
            foreach (BtsApplication application in catalog.Applications)
                if (application.IsSystem)
                    return application;

            throw new BtsException(Resources.BizTalkMgmt_SystemAppNotFound);
        }

        public static XmlDocument GetResourceSpec(BtsApplication application)
        {
            BtsResourceCollection collection = new BtsResourceCollection(application);

            Microsoft.BizTalk.ApplicationDeployment.ResourceSpec spec = new Microsoft.BizTalk.ApplicationDeployment.ResourceSpec();
            spec.ApplicationName = application.Name;
            spec.Resources = new Microsoft.BizTalk.ApplicationDeployment.ResourceSpecItem[collection.Count];

            int index = 0;
            foreach (BtsResource resource in collection)
            {
                Microsoft.BizTalk.ApplicationDeployment.ResourceSpecItem item = new Microsoft.BizTalk.ApplicationDeployment.ResourceSpecItem();
                item.Type = resource.ResourceType;
                item.Luid = resource.Luid;
                if ((String.Compare(resource.ResourceType, "System.BizTalk:WebDirectory", StringComparison.OrdinalIgnoreCase) == 0) && resource.Properties.ContainsKey("SourceLocation"))
                    item.Source = (string)resource.Properties["SourceLocation"];
                spec.Resources[index] = item;
                index++;
            }

            // return the resulting Xml representation

            string tempfile = Path.GetTempFileName();
            spec.SaveXml(tempfile);

            XmlDocument document = new XmlDocument();
            document.Load(tempfile);
            return document;
        }

        public static void ResetPipelines(BtsApplication application)
        {
            BtsApplication systemApplication = GetSystemApplication(application.Catalog);
            BtsPipeline defaultReceivePipeline = systemApplication.Pipelines[BtsPipelineHelper.PassThruReceivePipeline];
            BtsPipeline defaultSendPipeline = systemApplication.Pipelines[BtsPipelineHelper.PassThruTransmitPipeline];
        
            foreach (BtsReceivePort receivePort in application.ReceivePorts)
            {
                foreach (BtsReceiveLocation receiveLocation in receivePort.ReceiveLocations)
                {
                    receiveLocation.ReceivePipeline = defaultReceivePipeline;
                    if (receivePort.IsTwoWay)
                        receiveLocation.SendPipeline = defaultSendPipeline;
                }
            }

            foreach (BtsSendPort sendPort in application.SendPorts)
            {
                sendPort.SendPipeline = defaultSendPipeline;
                if (sendPort.IsTwoWay)
                    sendPort.ReceivePipeline = defaultReceivePipeline;
            }

            application.Catalog.SaveChanges();
        }

        public static void RemoveAllReceivePorts(BtsApplication application)
        {
            foreach (BtsReceivePort receivePort in application.ReceivePorts)
                application.RemoveReceivePort(receivePort);
            application.Catalog.SaveChanges();
        }

        public static void RemoveAllSendPortGroups(BtsApplication application)
        {
            foreach (BtsSendPortGroup sendPortGroup in application.SendPortGroups)
                application.RemoveSendPortGroup(sendPortGroup);
            application.Catalog.SaveChanges();
        }

        public static void RemoveAllSendPorts(BtsApplication application)
        {
            foreach (BtsSendPort sendPort in application.SendPorts)
                application.RemoveSendPort(sendPort);
            application.Catalog.SaveChanges();
        }

        public static void UninstallApplication(string name)
        {
            UninstallApplication(name, Helper.MsiOptions.Passive);
        }

        public static void UninstallApplication(string name, Helper.MsiOptions options)
        {
            UninstallApplication(name, options, null);
        }

        public static void UninstallApplication(string name, Helper.MsiOptions options, string logfile)
        {
            RegistryKey hKeyUninst = null;
            RegistryKey hKeyApp = null;

            try
            {
                // check UninstallString in Add/Remove Programs
                // for a string containing "BtsTask.exe UninstallApp"

                hKeyUninst = Registry.LocalMachine.OpenSubKey(@"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall", RegistryKeyPermissionCheck.ReadSubTree);
                hKeyApp = hKeyUninst.OpenSubKey(name, RegistryKeyPermissionCheck.ReadSubTree);

                if (hKeyApp == null)
                    throw new BtsException(String.Format(Resources.BizTalkMgmt_InvalidAddRemoveAppName, name));

                if (!(hKeyApp.GetValue("UninstallString") as string).ToLower().Contains("btstask.exe uninstallapp"))
                    throw new BtsException(String.Format(Resources.BizTalkMgmt_InvalidAddRemoveBtsApp, name));

                // a subkey contains the GUID of the Windows Installer package

                string[] subkeys = hKeyApp.GetSubKeyNames();
                foreach (string subkey in subkeys)
                {
                    Guid identifier = Guid.Empty;

                    try { identifier = new Guid(subkey); }
                    catch { identifier = Guid.Empty; }

                    if (identifier.CompareTo(Guid.Empty) != 0)
                        Helper.MsiHelper.MsiExecUninst(identifier, options, logfile);
                }
            }

            catch (BtsException /* e */) { throw; }
            catch (System.Exception e) { throw new BtsException(e); }

            finally
            {
                if (hKeyApp != null)
                    hKeyApp.Close();
                if (hKeyUninst != null)
                    hKeyUninst.Close();
            }
        }
    }
}
