﻿using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.RuleEngine;
using Microsoft.BizTalk.RuleEngineExtensions;

using RuleSetDeploymentDriver = Microsoft.BizTalk.RuleEngineExtensions.RuleSetDeploymentDriver;

// TODO: add option to remove associated vocabularies when removing policies

namespace BizTalkFactory.Management.Automation.Helper
{
    public static class BtsPolicyHelper
    {
        internal static string BizTalkRuleEngineDb = "BizTalkRuleEngineDb";
        internal static string BizTalkruleEngineRunningServer = Environment.MachineName;

        public static void ImportAndPublish(string source, out RuleSetInfoCollection ruleSets, out VocabularyInfoCollection vocabularies)
        {
            try
            {
                ruleSets = new RuleSetInfoCollection();
                vocabularies = new VocabularyInfoCollection();

                RuleSetDeploymentDriver driver = new RuleSetDeploymentDriver();
                SqlRuleStore sqlRuleStore = (SqlRuleStore)driver.GetRuleStore();

                FileRuleStore fileRuleStore = new FileRuleStore(source);

                // import vocabularies first

                VocabularyInfoCollection vocabularyInfos = fileRuleStore.GetVocabularies(RuleStore.Filter.All);
                foreach (VocabularyInfo vocabularyInfo in vocabularyInfos)
                {
                    vocabularies.Add(vocabularyInfo);

                    Vocabulary vocabulary = fileRuleStore.GetVocabulary(vocabularyInfo);
                    sqlRuleStore.Add(vocabulary, true);
                }

                // import policies

                RuleSetInfoCollection ruleSetInfos = fileRuleStore.GetRuleSets(RuleStore.Filter.All);
                foreach (RuleSetInfo ruleSetInfo in ruleSetInfos)
                {
                    ruleSets.Add(ruleSetInfo);

                    RuleSet ruleset = fileRuleStore.GetRuleSet(ruleSetInfo);
                    sqlRuleStore.Add(ruleset, true);
                }

                // the following seems redundant ?
                //driver.ImportAndPublishFileRuleStore(source);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public static void Export(string name, string destination)
        {
            Export(name, destination, true);
        }

        public static void Export(string name, string version, string destination)
        {
            Export(String.Format("{0}:{1}", name, version), destination, true);
        }

        public static void Export(string name, string destination, bool includeReferencedVocabularies)
        {
            try
            {
                RuleSetDeploymentDriver driver = new RuleSetDeploymentDriver();
                SqlRuleStore sqlRuleStore = (SqlRuleStore)driver.GetRuleStore();

                // export specified policy by matching name or version

                RuleSetDictionary ruleSets = MatchRuleSetsByNameOrVersion(sqlRuleStore, name);
                VocabularyDictionary vocabularies = new VocabularyDictionary();

                // export referenced vocabularies

                if (includeReferencedVocabularies)
                {
                    vocabularies = GetReferencedVocabularies(sqlRuleStore, ruleSets);
                    vocabularies = GetReferencedVocabularies(sqlRuleStore, vocabularies);
                }

                // export specified policies and vocabularies to file

                RuleStore fileRuleStore = new FileRuleStore(destination);
                fileRuleStore.Save(ruleSets, vocabularies, false);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public static bool IsDeployed(RuleSetInfo info)
        {
            try
            {
                RuleSetDeploymentDriver driver = new RuleSetDeploymentDriver();
                return driver.IsRuleSetDeployed(info);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public static void Deploy(RuleSetInfo info)
        {
            try
            {
                RuleSetDeploymentDriver driver = new RuleSetDeploymentDriver();
                driver.Deploy(info);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public static void Undeploy(RuleSetInfo info)
        {
            try
            {
                RuleSetDeploymentDriver driver = new RuleSetDeploymentDriver();
                driver.Undeploy(info);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public static void Remove(RuleSetInfo info)
        {
            try
            {
                RuleSetDeploymentDriver driver = new RuleSetDeploymentDriver();
                SqlRuleStore ruleStore = (SqlRuleStore)driver.GetRuleStore();

                ruleStore.Remove(info);
            }

            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public static RuleSetInfo ParseResourceLuid(string luid)
        {
            string[] components = luid.Split(new char[] { '/' });

            if (components.Length != 3)
                throw new BtsException(String.Format(Resources.BizTalkMgmt_InvalidRuleSetLuid, luid));

            try
            {
                string name = components[1];
                string version = components[2];

                int major = Int32.Parse(version.Split(new char[] { '.' })[0]);
                int minor = Int32.Parse(version.Split(new char[] { '.' })[1]);

                return new RuleSetInfo(name, major, minor);
            }

            catch (System.FormatException e)
            { throw new BtsException(String.Format(Resources.BizTalkMgmt_InvalidRuleSetLuid, luid), e); }
            catch (System.Exception e)
            { throw new BtsException(e); }
        }

        public static RuleSetInfo GetRuleSetInfoFromLuid(string luid)
        {
            RuleSetInfo ruleSetInfo = ParseResourceLuid(luid);
            return GetRuleSetInfoFromName(ruleSetInfo.Name, ruleSetInfo.MajorRevision, ruleSetInfo.MinorRevision);
        }

        public static RuleSetInfo GetRuleSetInfoFromName(string name, int major, int minor)
        {
            return GetRuleSetInfoFromName(name, String.Format("{0}.{1}", major, minor));
        }

        public static RuleSetInfo GetRuleSetInfoFromName(string name, string version)
        {
            RuleSetDeploymentDriver driver = new RuleSetDeploymentDriver();
            SqlRuleStore sqlRuleStore = (SqlRuleStore)driver.GetRuleStore();

            List<RuleSetInfo> ruleSetInfos = MatchRuleSetInfosByNameOrVersion(sqlRuleStore, name + ":" + version);
            foreach (RuleSetInfo ruleSetInfo in ruleSetInfos)
                return ruleSetInfo;

            throw new KeyNotFoundException(String.Format(Resources.BizTalkMgmt_PolicyNotFound, name, version));
        }

        #region Implementation

        private static RuleSetDictionary MatchRuleSetsByNameOrVersion(SqlRuleStore ruleStore, string name)
        {
            RuleSetDictionary ruleSets = new RuleSetDictionary();

            List<RuleSetInfo> ruleSetInfos = MatchRuleSetInfosByNameOrVersion(ruleStore, name);
            foreach (RuleSetInfo ruleSetInfo in ruleSetInfos)
                ruleSets.Add(ruleStore.GetRuleSet(ruleSetInfo));

            return ruleSets;
        }

        private static List<RuleSetInfo> MatchRuleSetInfosByNameOrVersion(SqlRuleStore ruleStore, string name)
        {
            List<RuleSetInfo> resultingList = new List<RuleSetInfo>(1);

            // obtain specified policy name and version parts

            string policyName = name;
            string version = String.Empty;

            int colon = name.LastIndexOf(':');
            if (colon > 0)
            {
                policyName = name.Substring(0, colon);
                version = name.Substring(colon + 1);
            }

            string major = version;
            string minor = String.Empty;

            int dot = version.LastIndexOf('.');
            if (dot > 0)
            {
                major = version.Substring(0, dot);
                minor = version.Substring(dot + 1);
            }

            int majorRevision = (String.IsNullOrEmpty(major) ? 0 : Int32.Parse(major));
            int minorRevision = (String.IsNullOrEmpty(minor) ? 0 : Int32.Parse(minor));

            // match name and/or version

            RuleSetInfoCollection ruleSetInfos = ruleStore.GetRuleSets(policyName, RuleStore.Filter.All);
            foreach (RuleSetInfo ruleSetInfo in ruleSetInfos)
            {
                // if version is not specified, return them all

                if (String.IsNullOrEmpty(version))
                    resultingList.Add(ruleSetInfo);

                // otherwise, if minor version is not specified, check major version number

                else if (String.IsNullOrEmpty(minor) && (ruleSetInfo.MajorRevision == majorRevision))
                    resultingList.Add(ruleSetInfo);

                // otherwise, check specific version

                else if (ruleSetInfo.MajorRevision == majorRevision && ruleSetInfo.MinorRevision == minorRevision)
                    resultingList.Add(ruleSetInfo);
            }

            return resultingList;
        }

        private static VocabularyDictionary GetReferencedVocabularies(SqlRuleStore ruleStore, RuleSetDictionary ruleSets)
        {
            // get referenced vocabularies from specified rulesets

            List<VocabularyInfo> referencedVocabularyInfos = new List<VocabularyInfo>();

            foreach (RuleSet ruleset in ruleSets.Values)
            {
                RuleSetInfo ruleSetInfo = new RuleSetInfo(ruleset.Name,
                    ruleset.CurrentVersion.MajorRevision,
                    ruleset.CurrentVersion.MinorRevision);

                VocabularyInfoCollection vocabularyInfos = ruleStore.GetReferencedVocabularies(ruleSetInfo);
                foreach (VocabularyInfo vocabularyInfo in vocabularyInfos)
                {
                    if (!referencedVocabularyInfos.Contains(vocabularyInfo))
                        referencedVocabularyInfos.Add(vocabularyInfo);
                }
            }

            // return resulting list of referenced vocabularies

            VocabularyDictionary referencedVocabularies = new VocabularyDictionary();
            foreach (VocabularyInfo vocabularyInfo in referencedVocabularyInfos)
            {
                Vocabulary vocabulary = ruleStore.GetVocabulary(vocabularyInfo);
                if (!IsBuiltinVocabulary(vocabulary))
                    referencedVocabularies.Add(vocabulary);
            }

            return referencedVocabularies;
        }

        private static VocabularyDictionary GetReferencedVocabularies(SqlRuleStore ruleStore, VocabularyDictionary vocabularies)
        {
            List<VocabularyInfo> referencedVocabularyInfos = new List<VocabularyInfo>(vocabularies.Count);

            // add already specified vocabularies

            foreach (Vocabulary vocabulary in vocabularies.Values)
            {
                VocabularyInfo vocabularyInfo = new VocabularyInfo(vocabulary.Name,
                    vocabulary.CurrentVersion.MajorRevision,
                    vocabulary.CurrentVersion.MinorRevision);

                if (!referencedVocabularyInfos.Contains(vocabularyInfo))
                    referencedVocabularyInfos.Add(vocabularyInfo);
            }

            // get referenced vocabularies from specified vocabularies

            foreach (Vocabulary vocabulary in vocabularies.Values)
            {
                VocabularyInfo vocabularyInfo = new VocabularyInfo(vocabulary.Name,
                    vocabulary.CurrentVersion.MajorRevision,
                    vocabulary.CurrentVersion.MinorRevision);

                VocabularyInfoCollection vocabularyInfos = ruleStore.GetReferencedVocabularies(vocabularyInfo);
                foreach (VocabularyInfo info in vocabularyInfos)
                    if (!referencedVocabularyInfos.Contains(info))
                        referencedVocabularyInfos.Add(info);
            }

            // add nested dependencies until not more
            // (we add to the list as part of the loop)

            for (int i = 0; i < referencedVocabularyInfos.Count; i++)
            {
                VocabularyInfo vocabularyInfo = referencedVocabularyInfos[i];

                VocabularyInfoCollection vocabularyInfos = ruleStore.GetReferencedVocabularies(vocabularyInfo);
                foreach (VocabularyInfo info in vocabularyInfos)
                    if (!referencedVocabularyInfos.Contains(info))
                        referencedVocabularyInfos.Add(info);
            }

            // get resulting list of referenced vocabularies

            VocabularyDictionary referencedVocabularies = new VocabularyDictionary();
            foreach (VocabularyInfo vocabularyInfo in referencedVocabularyInfos)
            {
                Vocabulary vocabulary = ruleStore.GetVocabulary(vocabularyInfo);
                if (!IsBuiltinVocabulary(vocabulary))
                    referencedVocabularies.Add(vocabulary);
            }

            return referencedVocabularies;
        }

        // taken from BtsTask.exe
        private static bool IsBuiltinVocabulary(Vocabulary vocabulary)
        {
            // no easy way to tell what is built-in since they might be localized
            const string PredicatesVocabularyId = "3f0e9bcc-6212-4e6a-853c-e517f157a626";
            const string FunctionsVocabularyId = "aee857f2-09d5-4615-a96c-87041791ec42";
            const string CommonSetsVocabularyId = "5e19cad8-b5f1-4f77-93fd-5210c3e45394";
            const string CommonValuesVocabularyId = "cfded0c3-02bc-4403-ae44-ffdeeb2297d8";

            string vocabId = vocabulary.Id;

            return ((string.Compare(vocabId, FunctionsVocabularyId, StringComparison.OrdinalIgnoreCase) == 0) ||
                    (string.Compare(vocabId, PredicatesVocabularyId, StringComparison.OrdinalIgnoreCase) == 0) ||
                    (string.Compare(vocabId, CommonSetsVocabularyId, StringComparison.OrdinalIgnoreCase) == 0) ||
                    (string.Compare(vocabId, CommonValuesVocabularyId, StringComparison.OrdinalIgnoreCase) == 0));
        }

        #endregion
    }
}
