using System;
using System.Collections.ObjectModel;
using System.Management.Automation;
using System.Management.Automation.Provider;
using System.Globalization;

namespace BizTalkFactory.PowerShell.Extensions
{
    [CmdletProvider("BizTalk", ProviderCapabilities.None)]
    public class BizTalkProvider : NavigationCmdletProvider, IPropertyCmdletProvider, ICmdletProviderSupportsHelp
    {
        #region private constants

        private const string ERROR_RESOLVE_PROPERTY = "Property name '{0}' doesn't exist for item at path '{1}'.";

        #endregion

        // CmdletProvider is a base class for a provider that hooks into the
        // namespace. This class is at the top of the hierarchy of all 
        // classes that can be used to derive from to create a new 
        // Windows PowerShell provider. Although it is possible to create a 
        // CmdletProvider by deriving from this class, developers are strongly 
        // encouraged to derive from either of ItemCmdletProvider, 
        // ContainerCmdletProvider or NavigationCmdletProvider. 

        #region CmdletProvider Overrides

        #endregion

        // DriveCmdletProvider is a base class for a provider that hooks into
        // the Windows PowerShell namespace and that can be exposed
        // through Windows PowerShell drives. A drive name can be a valid sequence of 
        // alpha numneric characters. Although it is possible to create a
        // CmdletProvider that can be exposed through a drive by directly
        // deriving from this class, developers are strongly encouraged to
        // derive from either of ItemCmdletProvider, ContainerCmdletProvider
        // or NavigationCmdletProvider.

        #region DriveCmdletProvider Overrides

        /// <summary>
        /// Gives the provider an opportunity to validate the drive that is
        /// being added.  It also allows the provider to modify parts of the
        /// PSDriveInfo object.  This may be done for performance or
        /// reliability reasons or to provide extra data to all calls using
        /// the Drive.
        /// </summary>
        /// 
        /// <param name="drive">
        /// The proposed new drive.
        /// </param>
        /// 
        /// <returns>
        /// The new drive that is to be added to the Windows PowerShell namespace.  This
        /// can either be the same <paramref name="drive"/> object that
        /// was passed in or a modified version of it. The default 
        /// implementation returns the drive that was passed.
        /// </returns>
        /// 
        /// <remarks>
        /// This method gives the provider an opportunity to associate 
        /// provider specific information with a drive. This is done by 
        /// deriving a new class from 
        /// <see cref="System.Management.Automation.PSDriveInfo"/>
        /// and adding any properties, methods, or fields that are necessary. 
        /// When this method gets called, the override should create an instance
        /// of the derived PSDriveInfo using the passed in PSDriveInfo. The derived
        /// PSDriveInfo should then be returned. Each subsequent call into the provider
        /// that uses this drive will have access to the derived PSDriveInfo via the
        /// PSDriveInfo property provided by the base class. Implementers of this 
        /// method should verify that the root exists and that a connection to 
        /// the data store (if there is one) can be made.  Any failures should 
        /// be sent to the
        /// <see cref="System.Management.Automation.Provider.CmdletProvider.WriteError(ErrorRecord)"/>
        /// method and null should be returned.
        /// </remarks>
        protected override PSDriveInfo NewDrive(PSDriveInfo drive)
        {
            if (drive == null)
            {
                WriteError(new ErrorRecord(
                              new ArgumentNullException("drive")
                            , "NullDrive"
                            , ErrorCategory.InvalidArgument
                            , drive));
                return null;
            }

            if (drive.Root == null)
            {
                WriteError(new ErrorRecord(
                              new ArgumentNullException("drive.Root")
                            , "NullRoot"
                            , ErrorCategory.InvalidArgument
                            , drive));
                return null;
            }

            try
            {
                // If the drive has already been initialized by InitializeDefaultDrives().
                // If so return the drive.
                if (drive is BizTalkDriveInfo)
                {
                    return drive;
                }

                BizTalkDriveInfo driveInfo = new BizTalkDriveInfo(this, drive, (BizTalkDriveParameters)DynamicParameters);

                return driveInfo;
            }
            catch (Exception e)
            {
                ThrowTerminatingError(
                    new ErrorRecord(
                          e
                        , "0"
                        , ErrorCategory.InvalidArgument
                        , drive));
            }

            return null;
        }

        /// <summary>
        /// Cleans up provider specific data for a drive before it is  
        /// removed. This method gets called before a drive gets removed. 
        /// </summary>
        /// 
        /// <param name="drive">
        /// The Drive object that represents the mounted drive.
        /// </param>
        /// 
        /// <returns>
        /// If the drive can be removed then the drive that was passed in is 
        /// returned. If the drive cannot be removed, null should be returned
        /// and an exception is written to the 
        /// <see cref="System.Management.Automation.Provider.CmdletProvider.WriteError(ErrorRecord)"/>
        /// method. The default implementation returns the drive that was 
        /// passed.
        /// </returns>
        /// 
        /// <remarks>
        /// An implementer has to ensure that this method is overridden to free 
        /// any resources that may be associated with the drive being removed.
        /// </remarks>
        protected override PSDriveInfo RemoveDrive(PSDriveInfo drive)
        {
            if (drive == null)
            {
                WriteError(new ErrorRecord(
                              new ArgumentNullException("drive")
                            , "NullDrive"
                            , ErrorCategory.InvalidArgument
                            , drive));
                return null;
            }

            return drive;
        }

        /// <summary>
        /// Allows the provider to map drives after initialization.
        /// </summary>
        /// 
        /// <returns>
        /// A drive collection with the drives that the provider wants to be 
        /// added to the session upon initialization. The default 
        /// implementation returns an empty 
        /// <see cref="System.Management.Automation.PSDriveInfo"/> collection.
        /// </returns>
        /// 
        /// <remarks>
        /// After the Start method is called on a provider, the
        /// InitializeDefaultDrives method is called. This is an opportunity
        /// for the provider to mount drives that are important to it. For
        /// instance, the Active Directory provider might mount a drive for
        /// the defaultNamingContext if the machine is joined to a domain.
        /// 
        /// All providers should mount a root drive to help the user with
        /// discoverability. This root drive might contain a listing of a set
        /// of locations that would be interesting as roots for other mounted
        /// drives. For instance, the Active Directory provider may create a
        /// drive that lists the naming contexts found in the namingContext
        /// attributes on the RootDSE. This will help users discover
        /// interesting mount points for other drives.
        /// </remarks>
        protected override Collection<PSDriveInfo> InitializeDefaultDrives()
        {
            // Find out if initialization of default drive needs to be skipped.
            // This is useful when the user has client tools only or has named
            // the BizTalk management database differently.
            // Skipping can be done by setting the a boolean session variable
            // called '$InitializeDefaultBTSDrive'. This must set before
            // adding the snap in:
            //      $InitializeDefaultBTSDrive = $true
            //      Add-PSSnapin BizTalkFactory.PowerShell.Extensions
            // If the session variable is not provided or of the wrong type
            // the default drives are not skipped.
            if (SkipInitializeDefaultDrives())
            {
                return null;
            }

            Collection<PSDriveInfo> drives = new Collection<PSDriveInfo>();
            PSDriveInfo psDriveInfo = new PSDriveInfo("BizTalk", ProviderInfo, @"Biztalk:\", String.Empty, null);
            drives.Add(new BizTalkDriveInfo(this, psDriveInfo, new BizTalkDriveParameters()));
            return drives;
        }

        

        /// <summary>
        /// Allows the provider to attach additional parameters to the 
        /// New-PSDrive cmdlet.
        /// </summary>
        /// 
        /// <returns>
        /// Implementors of this method should return an object that has 
        /// properties and fields decorated with parsing attributes similar 
        /// to a cmdlet class or a 
        /// <see cref="System.Management.Automation.PseudoParameterDictionary"/>.
        /// The default implemenation returns null.
        /// </returns>
        protected override object NewDriveDynamicParameters()
        {
            return new BizTalkDriveParameters();
        }

        #endregion DriveCmdletProvider Overrides

        // ItemCmdletProvider is a base class for a provider of a single item, 
        // that hooks into the Windows PowerShell namespace and which 
        // exposes the item as an Windows PowerShell path. When inherited from this 
        // class, the provider inherits a set of methods that allows the Windows 
        // PowerShell engine to provide a core set of commands for getting 
        // and setting of data on one or more items. A provider should derive 
        // from this class if they want to take advantage of the item core 
        // commands that are already implemented by the engine. This allows 
        // users to have common commands and semantics across multiple 
        // providers.

        #region ItemCmdletProvider Overrides

        /// <summary>
        /// Determines if the specified path is syntactically and semantically
        /// valid.
        /// </summary>
        /// 
        /// <param name="path">
        /// The path to validate.
        /// </param>
        /// 
        /// <returns>
        /// True if the path is syntactically and semantically valid for the 
        /// provider, false otherwise. The default implementation checks if 
        /// the path is not null or empty.
        /// </returns>
        /// 
        /// <remarks>
        /// This test should not verify the existance of the item at the path. 
        /// It should only perform syntactic and semantic validation of the 
        /// path. For instance, for the file system provider, that path should
        /// be canonicalized, syntactically verified, and ensure that the path
        /// does not refer to a device.
        /// </remarks>
        protected override bool IsValidPath(string path)
        {
            path = MakePath(path, String.Empty);
            foreach (string item in path.Split(new char[] { '\\' }))
                if (item.Length == 0)
                    return false;

            return true;
        }

        /// <summary>
        /// Determines if an item exists at the specified path.
        /// </summary>
        /// <param name="path">
        /// The path to the item to see if it exists. 
        /// </param>
        /// <returns>
        /// True if the item exists, false otherwise.
        /// </returns>
        /// 
        /// <remarks>
        /// Providers can override this method to give the user the ability to 
        /// check for the existence of provider objects using the test-path 
        /// cmdlet. The default implementation returns false.
        /// 
        /// Providers that declare 
        /// <see cref="System.Management.Automation.Provider.ProviderCapabilities"/>
        /// of ExpandWildcards, Filter, Include, or Exclude should ensure that 
        /// the path passed meets those requirements by accessing the 
        /// appropriate property from the base class.
        /// 
        /// The implemenation of this method should take into account any form 
        /// of access to the object that may make it visible to the user.  For 
        /// instance, if a user has write access to a file in the file system 
        /// provider but not read access, the file still exists and the method
        /// should return true.  Sometimes this may require checking the parent
        /// to see if the child can be enumerated.
        /// </remarks>
        protected override bool ItemExists(string path)
        {
            return ItemExistsInternal(path);
        }

        /// <summary>
        /// Gets the item at the specified path. 
        /// </summary>
        /// 
        /// <param name="path">
        /// The path to the item to retrieve.
        /// </param>
        /// 
        /// <returns>
        /// Nothing is returned, but all objects will be written to the 
        /// WriteItemObject method.
        /// </returns>
        /// 
        /// <remarks>
        /// Providers can override this method to give the user access to the 
        /// provider objects using the get-item and get-childitem cmdlets. All
        /// objects should be written to the 
        /// <see cref="System.Management.Automation.Provider.CmdletProvider.WriteItemObject"/>
        /// method.
        /// 
        /// Providers that declare 
        /// <see cref="System.Management.Automation.Provider.ProviderCapabilities"/>
        /// of ExpandWildcards, Filter, Include, or Exclude should ensure that 
        /// the path passed meets those requirements by accessing the 
        /// appropriate property from the base class.
        /// 
        /// By default overrides of this method should not write objects that 
        /// are generally hidden from the user unless the Force property is set
        /// to true. For instance, the FileSystem provider should not call 
        /// WriteItemObject for hidden or system files unless the Force 
        /// property is set to true. 
        /// 
        /// The default implementation of this method does nothing
        /// </remarks> 
        protected override void GetItem(string path)
        {
            BizTalkObject item = GetItemInternal(path);
            WriteItemObject(item.Item, path, item.IsContainer);
        }

        protected override object GetItemDynamicParameters(string path)
        {

            return GetItemInternal(path).GetItemDynamicParameters;
        }

        /// <summary>
        /// Determines if the item at the specified path has children.
        /// </summary>
        /// 
        /// <param name="path">
        /// The path to the item to see if it has children.
        /// </param>
        /// 
        /// <returns>
        /// True if the item has children, false otherwise.
        /// </returns>
        /// 
        /// <remarks>
        /// Providers override this method to give the provider infrastructure 
        /// the ability to determine if a particular provider object has 
        /// children without having to retrieve all the child items.
        /// 
        /// For providers that are derived from 
        /// <see cref="ContainerCmdletProvider"/> class, if a null or empty 
        /// path is passed, the provider should consider any items in the data 
        /// store to be children and return true.
        /// 
        /// If this provider exposes a root that contains interesting mount
        /// points (as described in InitializeDefaultDrives) it should return
        /// true when null or String.Empty is passed. 
        /// </remarks>
        protected override bool HasChildItems(string path)
        {
            if (String.IsNullOrEmpty(path))
                return true;

            return (GetItemInternal(path).GetChildItems().Length > 0);

        }

        #endregion ItemCmdletProvider Overrides

        // ContainerCmdletProvider is a base class for a provider of a container
        // of items, that hooks into the Windows PowerShell namespace. 
        // This provider base class has methods which can be overriden to allow 
        // the use of a set of core commands against the objects that the 
        // provider gives access to. By deriving from this class users can take 
        // advantage of all the features of the ItemCmdletProvider as well as
        // use of wildcards on objects and the following commands:
        //     get-childitem
        //     rename-item
        //     new-item
        //     remove-item
        //     set-location
        //     push-location
        //     pop-location
        //     get-location -stack

        #region ContainerCmdletProvider Overrides

        /// <summary>
        /// Gets the children of the item at the specified path.
        /// </summary>
        /// 
        /// <param name="path">
        /// The path (or name in a flat namespace) to the item from which to
        /// retrieve the children.
        /// </param>
        /// 
        /// <param name="recurse">
        /// True if all children in a subtree should be retrieved, false if
        /// only a single level of children should be retrieved.  
        /// </param>
        /// 
        /// <remarks>
        /// Providers override this method to give the user access to the 
        /// provider objects using the get-childitem cmdlets.
        /// 
        /// Providers that declare 
        /// <see cref="System.Management.Automation.Provider.ProviderCapabilities"/>
        /// of ExpandWildcards, Filter, Include, or Exclude should ensure that 
        /// the path passed meets those requirements by accessing the 
        /// appropriate property from the base class.
        /// 
        /// By default, overrides of this method should not write objects that 
        /// are generally hidden from the user unless the Force property is set
        /// to true. For instance, the FileSystem provider should
        /// not call WriteItemObject for hidden or system files unless the Force
        /// property is set to true. 
        /// 
        /// The provider implementation is responsible for preventing infinite 
        /// recursion when there are circular links and the like. An 
        /// appropriate terminating exception should be thrown if this
        /// situation occurs.
        /// </remarks>
        protected override void GetChildItems(string path, bool recurse)
        {
            BizTalkObject[] items = GetChildItemsInternal(path, recurse);
            foreach (BizTalkObject item in items)
                WriteItemObject(item.Item, MakePath(path, item.Name), item.IsContainer);

        }

        /// <summary>
        /// Gets names of the children of the specified path.
        /// </summary>
        /// 
        /// <param name="path">
        /// The path to the item from which to retrieve the child names.
        /// </param>
        /// 
        /// <param name="returnAllContainers">
        /// If true, the provider should return all containers, even if they
        /// don't match the filter.  If false, the provider should only return
        /// containers that do match the filter.
        /// </param>
        /// 
        /// <remarks>
        /// Providers override this method to give the user access to the 
        /// provider objects using the get-childitem -name cmdlet.
        /// 
        /// Providers that declare 
        /// <see cref="System.Management.Automation.Provider.ProviderCapabilities"/>
        /// of ExpandWildcards, Filter, Include, or Exclude should ensure that
        /// the path passed meets those requirements by accessing the 
        /// appropriate property from the base class. The exception to this is
        /// if <paramref name="returnAllContainers"/> is true, then any child 
        /// name for a container should be returned even if it doesn't match 
        /// the Filter, Include, or Exclude.
        /// 
        /// By default overrides of this method should not write the names of
        /// objects that are generally hidden from the user unless the Force 
        /// property is set to true. For instance, the FileSystem provider 
        /// should not call WriteItemObject for hidden or system files unless 
        /// the Force property is set to true. 
        /// 
        /// The provider implementation is responsible for preventing infinite 
        /// recursion when there are circular links and the like. An 
        /// appropriate terminating exception should be thrown if this
        /// situation occurs.
        /// 
        /// The child names are the leaf portion of the path. Example, for the
        /// file system the name for the path c:\windows\system32\foo.dll would
        /// be foo.dll or for the directory c:\windows\system32 would be
        /// system32.  For Active Directory the child names would be RDN values
        /// of the child objects of the container.
        /// 
        /// All names should be written to the WriteItemObject method.
        /// </remarks>
        protected override void GetChildNames(string path, ReturnContainers returnContainers)
        {
            BizTalkObject[] items = GetChildItemsInternal(path, false);
            foreach (BizTalkObject item in items)
                WriteItemObject(item.Name, MakePath(path, item.Name), item.IsContainer);

        }

        protected override void NewItem(string path, string itemTypeName, object newItemValue)
        {
            string parent = GetParentPath(path, String.Empty);
            string name = GetChildName(path);

            BizTalkObject item = GetItemInternal(parent);
            BizTalkObject newItem = item.NewItem(name, itemTypeName, DynamicParameters);

            WriteItemObject(newItem.Item, MakePath(parent, name), newItem.IsContainer);
        }

        protected override object NewItemDynamicParameters(string path, string itemTypeName, object newItemValue)
        {
            // itemTypeName is ignored
            // newItemValue is ignored

            return GetItemInternal(GetParentPath(path, String.Empty)).NewItemDynamicParameters;
        }

        protected override void RemoveItem(string path, bool recurse)
        {
            GetItemInternal(path).RemoveItem(recurse);
        }

        #endregion ContainerCmdletProvider Overrides

        // NavigationCmdletProvider is a base class for a provider of a tree of 
        // items, that hooks into the Windows PowerShell namespace. By 
        // deriving from this class users can take advantage the recursive 
        // commands, nested containers, and relative paths.

        #region NavigationCmdletProvider Overrides

        /// <summary>
        /// Determines if the specified object is a container
        /// </summary>
        /// 
        /// <param name="path">
        /// The path to the item to determine if it is a container.
        /// </param>
        /// 
        /// <returns>
        /// true if the item specified by path exists and is a container, 
        /// false otherwise.
        /// </returns>
        /// 
        /// <remarks>
        /// Providers override this method to give the user the ability to 
        /// check to see if a provider object is a container using the 
        /// test-path -container cmdlet.
        /// 
        /// Providers that declare 
        /// <see cref="System.Management.Automation.Provider.ProviderCapabilities"/>
        /// of ExpandWildcards, Filter, Include, or Exclude should ensure that 
        /// the path passed meets those requirements by accessing the 
        /// appropriate property from the base class.
        /// </remarks>
        protected override bool IsItemContainer(string path)
        {
            return GetItemInternal(path).IsContainer;
        }

        /// <summary>
        /// Gets the name of item to which the path parameter points.
        /// In our case this is just the leaf of the path. Everything after the last '\'
        /// character.
        /// </summary>
        /// 
        /// <param name="path">
        /// The path to get the name from.
        /// </param>
        /// 
        /// <returns>
        /// The name of the item.
        /// </returns>
        /// 
        /// <remarks>
        /// This is almost the same implementation as the internal (base) class implemenation.
        /// We cannot use that here since it calls ItemExists and we would get in an infinite
        /// loop.
        /// </remarks>
        protected override string GetChildName(string path)
        {
            return PathUtil.GetChildName(path);
        }

        protected override string NormalizeRelativePath(string path, string basePath)
        {
            path = NormalizePath(path);
            return base.NormalizeRelativePath(path, basePath);
        }

        #endregion NavigationCmdletProvider Overrides

        // ICmdletProviderSupportsHelp is a base class for that
        // provides methods for displaying provider specific Help for items in the data store.

        #region ICmdletProviderSupportsHelp Members

        public string GetHelpMaml(string helpItemName, string path)
        {
            string helpFile = System.IO.Path.Combine(SnapIn.Location, SnapIn.HelpFileName);
            using (System.IO.StreamReader reader = new System.IO.StreamReader(helpFile))
                return reader.ReadToEnd();
        }

        #endregion

        #region IPropertyCmdletProvider Members

        public void ClearProperty(string path, Collection<string> propertyToClear)
        {
        }

        public object ClearPropertyDynamicParameters(string path, Collection<string> propertyToClear)
        {
            return null;
        }

        public void GetProperty(string path, Collection<string> providerSpecificPickList)
        {
            bool propertyFound = false;

            BizTalkObject item = GetItemInternal(path);

            PSObject psObject = PSObject.AsPSObject(item.Item);

            PSObject result = new PSObject();

            if (providerSpecificPickList.Count == 0)
            {
                foreach (PSPropertyInfo prop in psObject.Properties)
                {
                    providerSpecificPickList.Add(prop.Name);
                }
            }

            foreach (string name in providerSpecificPickList)
            {
                propertyFound = true;
                PSPropertyInfo prop = psObject.Properties[name];
                object value = null;
                if (prop != null)
                {
                    value = prop.Value;
                }
                else
                {
                    ErrorRecord errorRecord = new ErrorRecord(new ArgumentException(), null, ErrorCategory.InvalidArgument, null);
                    errorRecord.ErrorDetails = new ErrorDetails(String.Format(CultureInfo.InvariantCulture, ERROR_RESOLVE_PROPERTY, name, path));

                    WriteError(errorRecord);
                }
                result.Properties.Add(new PSNoteProperty(name, value));
            }

            if (propertyFound)
            {
                WritePropertyObject(result, path);
            }
        }

        public object GetPropertyDynamicParameters(string path, Collection<string> providerSpecificPickList)
        {
            return null;
        }

        public void SetProperty(string path, PSObject propertyValue)
        {
            BizTalkObject item = GetItemInternal(path);

            foreach (PSPropertyInfo propertyInfo in propertyValue.Properties)
            {
                // Validate if the property exists for the object layer object
                PSObject psObject = PSObject.AsPSObject(item.Item);
                PSPropertyInfo property = psObject.Properties[propertyInfo.Name];

                if (property == null)
                {
                    // The property is not a property of the object layer object
                    // See if the property exists for the wrapper object.
                    // Some properties do exist for from a powershell point of view
                    // while they do not directly map to a property of an object in the
                    // object layer.
                    psObject = PSObject.AsPSObject(item);
                    property = psObject.Properties[propertyInfo.Name];
                }

                if (property != null)
                {
                    try
                    {
                        item.SetPropertyValue(propertyInfo);
                    }
                    catch (ArgumentException argumentException)
                    {
                        ErrorRecord errorRecord = new ErrorRecord(argumentException, null, ErrorCategory.InvalidArgument, null);
                        WriteError(errorRecord);
                    }
                }
                else
                {
                    ErrorRecord errorRecord = new ErrorRecord(new ArgumentException(), null, ErrorCategory.InvalidArgument, null);
                    errorRecord.ErrorDetails = new ErrorDetails(String.Format(CultureInfo.InvariantCulture, ERROR_RESOLVE_PROPERTY, propertyInfo.Name, path));

                    WriteError(errorRecord);
                }
            }
        }

        public object SetPropertyDynamicParameters(string path, PSObject propertyValue)
        {
            return null;
        }

        #endregion

        #region Implementation

        private Catalog GetCatalogFromDrivePath(string path)
        {
            BizTalkDriveInfo bizTalkdrive = PathUtil.GetDrive(path, this.ProviderInfo) as BizTalkDriveInfo;

            if (bizTalkdrive == null)
            {
                ThrowTerminatingError(new ErrorRecord(new ArgumentException("The path doesn't include a valid BizTalk drive reference."), "", ErrorCategory.InvalidArgument, path));
            }

            Catalog catalog = new Catalog(bizTalkdrive.Catalog.Item, path);

            catalog.PSDriveInfo = PathUtil.GetDrive(path, this.ProviderInfo);

            return catalog;
        }

        internal BizTalkObject GetItemInternal(string path)
        {
            if (PathUtil.IsDrive(path, this.ProviderInfo))
            {
                return GetCatalogFromDrivePath(path);
            }

            string parent = base.GetParentPath(path, null);
            string name = GetChildName(path);

            BizTalkObject parentItem = GetItemInternal(parent);
            BizTalkObject item = parentItem.GetItem(name, DynamicParameters);

            item.PSDriveInfo = parentItem.PSDriveInfo;

            return item;
        }

        internal bool ItemExistsInternal(string path)
        {
            if (PathUtil.IsDrive(path, this.ProviderInfo))
                return true;

            string parent = base.GetParentPath(path, null);
            string name = GetChildName(path);

            if (!ItemExistsInternal(parent))
                return false;

            return GetItemInternal(parent).ItemExists(name);

        }

        internal BizTalkObject[] GetChildItemsInternal(string path, bool recurse)
        {
            return GetItemInternal(path).GetChildItems(recurse);
        }

        private string NormalizePath(string path)
        {
            if (String.IsNullOrEmpty(path))
                return path;

            string root = PathUtil.GetRoot(path, ProviderInfo);
            if (String.Compare(path, root, StringComparison.InvariantCultureIgnoreCase) == 0)
                return root;

            string parent = GetParentPath(path, root);
            string child = GetChildName(path);

            BizTalkObject[] items = GetChildItemsInternal(parent, false);
            foreach (BizTalkObject item in items)
                if (String.Compare(item.Name, child, StringComparison.InvariantCultureIgnoreCase) == 0)
                { child = item.Name; break; }

            if (!parent.EndsWith("\\"))
                parent += "\\";

            parent = NormalizePath(parent);

            if (!parent.EndsWith("\\"))
                parent += "\\";

            return parent + child;
        }

        /// <summary>
        /// Checks for a boolean session variable 'InitializeDefaultBTSDrive' and returns its value
        /// </summary>
        /// 
        /// <returns>
        /// True - if the session variable was set to $true
        /// False - In all other cases false is returned.
        /// </returns>
        private bool SkipInitializeDefaultDrives()
        {
            bool skip = false;

            if (SessionState != null)
            {
                PSVariable skipVariable = this.SessionState.PSVariable.Get("InitializeDefaultBTSDrive");

                if (skipVariable != null)
                {
                    object value = skipVariable.Value;

                    if (value != null)
                    {
                        if (value is bool)
                        {
                            skip = !(bool)value;
                        }
                        else
                        {
                            WriteError(new ErrorRecord(new ArgumentException("Variable 'InitializeDefaultBTSDrive' is not of type boolean. Initialization of default drive will not be skipped."), String.Empty, ErrorCategory.InvalidArgument, value));
                        }
                    }

                }
            }

            return skip;
        }

        #endregion
    }
}
