using System;
using System.Collections.ObjectModel;
using System.Management.Automation;
using System.Diagnostics;
using BizTalkFactory.Management.Automation;

namespace BizTalkFactory.PowerShell.Extensions.Cdmlets
{
    [Cmdlet("Export", "Bindings", DefaultParameterSetName = "PathParameterSet")]
    public class ExportBindingsCommand : BizTalkPathAndObjectCmdlet
    {
        // This cmdlet only supports a single path as input parameter because only
        // one destination parameter can be provided. 
        // Multiple paths would overwrite the destination file for each path that
        // doesn't make sense.

        private string destination_ = String.Empty;
        private bool include_party_bindings_ = false;
        private bool group_ = false;

        [Parameter(Position = 1, Mandatory = true)]
        [ValidateNotNullOrEmpty]
        public string Destination
        {
            get { return destination_; }
            set { destination_ = value; }
        }

        [Parameter(Position = 2)]
        public SwitchParameter GroupLevel
        {
            get { return group_; }
            set { group_ = value; }
        }

        [Parameter(Position = 3)]
        public SwitchParameter IncludeGlobalPartyBinding
        {
            get { return include_party_bindings_; }
            set { include_party_bindings_ = value; }
        }

        public ExportBindingsCommand()
            : base(true)
        { }

        protected override bool ValidateBtsType(object btsObject)
        {
            if (group_)
            {
                return true;
            }

            return (
                btsObject is BtsCatalog ||
                btsObject is BtsResource ||
                btsObject is BtsApplication);
        }

        protected override void ProcessItem(object btsObject)
        {
            try
            {
                if (group_)
                {
                    // When the group switch is provided we need to get the drive from the path 
                    // in order to know for which biztalk group we need to export bindings.
                    // We use SinglePath base class property with GetDrive method to get this information.
                    // This also means that if the group switch is provided the path can be anything
                    // it doesn't have to point to a drive only path like 'BizTalk:'
                    // Export-Bindings -Path 'BizTalk:\Applications\MyApplication\Send Ports' -GroupLevel -Destination c:\bindings.xml

                    BizTalkDriveInfo bizTalkDriveInfo = GetDrive(SinglePath) as BizTalkDriveInfo;
                    BtsCatalog btsCatalog = bizTalkDriveInfo.Catalog.Item as BtsCatalog;
                    btsCatalog.ExportBinding(destination_);
                }
                else
                {
                    if (btsObject is BtsCatalog)
                    {
                        // In this case the group switch is not provided but the path is pointing
                        // to a drive only like 'BizTalk:'
                        // Export-Bindings -Path 'BizTalk:' -Destination c:\bindings.xml
                        BtsCatalog btsCatalog = btsObject as BtsCatalog;
                        btsCatalog.ExportBinding(destination_);
                    }
                    else if (btsObject is BtsResource)
                    {
                        BtsResource btsResource = btsObject as BtsResource;
                        btsResource.ExportBinding(destination_, include_party_bindings_);
                    }
                    else if (btsObject is BtsApplication)
                    {
                        BtsApplication btsApplication = btsObject as BtsApplication;
                        btsApplication.ExportBinding(destination_, include_party_bindings_);
                    }
                }
            }
            catch (Exception e)
            {
                WriteError(new ErrorRecord(e, "0", ErrorCategory.InvalidArgument, btsObject));
            }
        }
    }
}
