using System;
using System.Management.Automation;
using System.Collections.ObjectModel;
using System.Collections.Generic;

namespace BizTalkFactory.PowerShell.Extensions
{
    internal sealed class PathUtil
    {
        public const char XML_PATH_SEP = '\\';
        public const char XML_DRIVE_SEP = ':';
        public const string XML_PROVIDER_SEP = "::";

        public readonly static char[] driveSeparator = new char[] { XML_DRIVE_SEP };            

        public static PSDriveInfo GetDrive(string path, ProviderInfo providerInfo)
        {
            // two formats possible
            // provider::drive:\path
            // drive:\path
            string[] paths = path.Split(driveSeparator, 2);
            string drivepath = paths[0];
            foreach (PSDriveInfo drive in providerInfo.Drives)
            {
                if (String.Compare(drive.Name, drivepath, StringComparison.OrdinalIgnoreCase) == 0)
                {
                    return drive;
                }
            }

            return null;
        }

        public static Collection<string> ResolvePath(SessionState sessionState, string path)
        {
            ProviderInfo providerInfo = null;
            return ResolvePath(sessionState, path, out providerInfo);
        }

        public static Collection<string> ResolvePath(SessionState sessionState, string path, out ProviderInfo providerInfo)
        {
            return sessionState.Path.GetResolvedProviderPathFromPSPath(path, out providerInfo);
        }

        public static PSDriveInfo GetDrive(SessionState sessionState, string path, out ProviderInfo providerInfo, out Collection<string> resolvedPaths)
        {
            providerInfo = null;

            resolvedPaths = PathUtil.ResolvePath(sessionState, path, out providerInfo);

            if (resolvedPaths == null)
            {
                return null;
            }

            if (String.IsNullOrEmpty(resolvedPaths[0]))
            {
                return null;
            }

            PSDriveInfo driveInfo = PathUtil.GetDrive(resolvedPaths[0], providerInfo);

            return driveInfo;
        }

        public static string GetRoot(string path, ProviderInfo providerInfo)
        {
            return PathUtil.GetDrive(path, providerInfo).Root;
        }

        public static string GetParentPath(string path)
        {
            path = Normalize(path);

            string parentPath = null;
            int separatorIndex = path.LastIndexOf('\\');
            if (separatorIndex != -1)
                parentPath = path.Substring(0, separatorIndex);

            return parentPath;
        }

        public static string GetChildName(string path)
        {
            path = Normalize(path);

            string childName = null;
            int separatorIndex = path.LastIndexOf('\\');
            if (separatorIndex == -1)
                childName = path;

            else
                childName = path.Substring(separatorIndex + 1);

            return childName;
        }

        public static bool IsDrive(string path, ProviderInfo providerInfo)
        {
            return (String.Compare(path, 0, GetRoot(path, providerInfo), 0, path.Length, StringComparison.OrdinalIgnoreCase) == 0);
        }

        public static string Normalize(string path)
        {
            path = path.Replace('/', XML_PATH_SEP);
            path = path.TrimEnd(new char[] { XML_PATH_SEP });

            return path;
        }

        public static object[] GetItems(PSDriveInfo driveInfo, Collection<string> resolvedPaths)
        {
            if (!(driveInfo is BizTalkDriveInfo))
            {
                return null;
            }

            BizTalkDriveInfo bizTalkDriveInfo = driveInfo as BizTalkDriveInfo;

            List<object> items = new List<object>();

            foreach (string resolvedPath in resolvedPaths)
            {
                items.Add(bizTalkDriveInfo.BizTalkProvider.GetItemInternal(resolvedPath).Item);
            }

            return items.ToArray();
        }
    }
}