﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Management.Automation;
using System.Globalization;
using BizTalkFactory.Management.Automation;
using System.Collections.ObjectModel;
using System.IO;

namespace BizTalkFactory.PowerShell.Extensions
{
    [AttributeUsage(AttributeTargets.Field | AttributeTargets.Property)]
    class ApplicationTransformationAttribute : ArgumentTransformationAttribute
    {
        #region private constants

        public const string ERROR_WRONG_TYPE = "At least one of the parameters could not be transformed to type 'BtsApplication'.";
        public const string ERROR_APPLICATION_NOT_FOUND = "Application '{0}' could not be found in the current catalog.";
        public const string ERROR_CONNECT_TO_DRIVE = "Unable to connect to biztalk drive object.";

        #endregion

        /// <summary>
        /// Transforms the object provided in 'inputData' to an array of BtsApplication elements 
        /// </summary>
        /// <param name="engineIntrinsics">Current engine intrinsics provided by the runt time</param>
        /// <param name="inputData">The input object provided to the cmd let as parameter </param>
        /// <returns>Array of BtsApplication elements. (BtsApplication[])</returns>
        public override object Transform(EngineIntrinsics engineIntrinsics, object inputData)
        {
            // This method will only transform inputData when it is one of the following types:
            //  - object[] where the elements are of type PsObject and base type of 'BtsApplication'
            //  - object[] where the elements are of type 'String'
            //  - System.String
            //  - PSObject with base object type BtsApplication.
            // All other types will cause an ParameterBindingException to be thrown.

            if (inputData is PSObject && 
                ((PSObject)inputData).BaseObject is BtsApplication)
            {
                return new BtsApplication[] { (BtsApplication)(((PSObject)inputData).BaseObject) };
            }
            else if (inputData is object[])  
            {
                object[] inputArray = (object[])inputData;

                // Check if all elements are of type BtsApplication.
                if (Array.TrueForAll(inputArray, delegate(object inputArrayElement)
                {
                    if (!(inputArrayElement is PSObject))
                    {
                        return false;
                    }
                    
                    return ((PSObject)inputArrayElement).BaseObject is BtsApplication;
                }))
                {
                    // convert array of psobjects to array of BtsApplications
                    return System.Array.ConvertAll<object, BtsApplication>(inputArray, delegate(object inputArrayElement)
                    {
                        return (BtsApplication)(((PSObject)inputArrayElement).BaseObject);
                    });
                }

                // Check if all elements are of type string.
                if (Array.TrueForAll(inputArray, delegate(object inputArrayElement)
                {
                    return inputArrayElement is System.String;
                }))
                {
                    // All the objects are strings i.e. contain application name.
                    // Now get BtsApplications by name.
                    return GetBtsApplications(engineIntrinsics, inputArray);
                }
            }
            else if (inputData is System.String)
            {
                // Object is single string containing application name.
                // Now get BtsApplications by name.
                string[] inputArray = new string[1] { (string)inputData };

                return GetBtsApplications(engineIntrinsics, (string[])inputArray);
            }
            
            throw new ParameterBindingException(ERROR_WRONG_TYPE);
        }

        private BtsApplication[] GetBtsApplications(EngineIntrinsics engineIntrinsics
            , object[] paths)
        {
            List<BtsApplication> btsApplicationList = new List<BtsApplication>();
            ProviderInfo providerInfo = null;

            foreach (string path in paths)
            {
                Collection<string> resolvedPaths = null;

                PSDriveInfo driveInfo = PathUtil.GetDrive(engineIntrinsics.SessionState, path, out providerInfo, out resolvedPaths);

                object[] applications = PathUtil.GetItems(driveInfo, resolvedPaths);

                foreach (object application in applications)
                {
                    if (application is BtsApplication)
                    {
                        btsApplicationList.Add(application as BtsApplication);
                    }
                }
            }

            return btsApplicationList.ToArray();
        }

        //private BtsApplication GetApplicationByName(BtsCatalog btsCatalog, string applicationName)
        //{
        //    foreach (BtsApplication btsApplication in btsCatalog.Applications)
        //    {
        //        if (btsApplication.Name.Equals(applicationName))
        //        {
        //            return btsApplication;
        //        }
        //    }

        //    throw new ParameterBindingException(String.Format(CultureInfo.InvariantCulture, ERROR_APPLICATION_NOT_FOUND, applicationName));
        //}

        //private BtsCatalog GetCatalog(EngineIntrinsics engineIntrinsics)
        //{
        //    BizTalkDriveInfo biztalkDrive = engineIntrinsics.SessionState.Drive.Current as BizTalkDriveInfo;

        //    if (biztalkDrive == null)
        //    {
        //        throw new ParameterBindingException(ERROR_CONNECT_TO_DRIVE);
        //    }

        //    BtsCatalog btsCatalog = BtsCatalog.Connect(biztalkDrive.Instance, biztalkDrive.Database);

        //    return btsCatalog;
        //}
    }
}
