﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Management.Automation;
using System.Globalization;
using System.IO;

namespace BizTalkFactory.PowerShell.Extensions
{
    [AttributeUsage(AttributeTargets.Field | AttributeTargets.Property)]
    class FileInfoTransformationAttribute : ArgumentTransformationAttribute
    {
        #region private constants

        public const string ERROR_WRONG_TYPE = "At least one of the parameters could not be transformed to type 'FileInfo'.";
        public const string ERROR_FILE_NOT_FOUND = "File '{0}' does not exist.";

        #endregion

        /// <summary>
        /// Transforms the object provided in 'inputData' to an array of FileInfo elements 
        /// </summary>
        /// <param name="engineIntrinsics">Current engine intrinsics provided by the runt time</param>
        /// <param name="inputData">The input object provided to the cmd let as parameter </param>
        /// <returns>Array of FileInfo elements. (FileInfo[])</returns>
        public override object Transform(EngineIntrinsics engineIntrinsics, object inputData)
        {
            // This method will only transform inputData when it is one of the following types:
            //  - object[] where the elements are of type PsObject and base type of 'FileInfo'
            //  - object[] where the elements are of type 'String'
            //  - System.String
            //  - PSObject with base object type FileInfo.
            // All other types will cause an ParameterBindingException to be thrown.

            if (inputData is PSObject && 
                ((PSObject)inputData).BaseObject is FileInfo)
            {
                return new FileInfo[] { (FileInfo)(((PSObject)inputData).BaseObject) };
            }
            else if (inputData is object[])  
            {
                object[] inputArray = (object[])inputData;

                // Check if all elements are of type FileInfo.
                if (Array.TrueForAll(inputArray, delegate(object inputArrayElement)
                {
                    if (!(inputArrayElement is PSObject))
                    {
                        return false;
                    }
                    
                    return ((PSObject)inputArrayElement).BaseObject is FileInfo;
                }))
                {
                    // convert array of psobjects to array of FileInfos
                    return System.Array.ConvertAll<object, FileInfo>(inputArray, delegate(object inputArrayElement)
                    {
                        return (FileInfo)(((PSObject)inputArrayElement).BaseObject);
                    });
                }

                // Check if all elements are of type string.
                if (Array.TrueForAll(inputArray, delegate(object inputArrayElement)
                {
                    return inputArrayElement is System.String;
                }))
                {
                    // All the objects are strings i.e. contain file name.
                    // Now get FileInfo from name.
                    return GetFileInfosFromName(engineIntrinsics, inputArray);
                }
            }
            else if (inputData is System.String)
            {
                // Object is single string containing file name.
                // Now get FileInfos by name.
                string[] inputArray = new string[1] { (string)inputData };

                return GetFileInfosFromName(engineIntrinsics, (string[])inputArray);
            }
            
            throw new ParameterBindingException(ERROR_WRONG_TYPE);
        }

        private FileInfo[] GetFileInfosFromName(EngineIntrinsics engineIntrinsics
            , object[] fileNames)
        {
            List<FileInfo> fileInfoList = new List<FileInfo>();

            foreach (string fileName in fileNames)
            {
                FileInfo fileInfo = new FileInfo(fileName);

                if (!fileInfo.Exists)
                {
                    throw new ParameterBindingException(String.Format(CultureInfo.InvariantCulture, ERROR_FILE_NOT_FOUND, fileName)); 
                }

                fileInfoList.Add(fileInfo);
            }

            return fileInfoList.ToArray();
        }
    }
}
